@@ -1,7 +1,7 @@
 // padthv1.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -37,6 +37,7 @@
 #include "padthv1_config.h"
 #include "padthv1_controls.h"
 #include "padthv1_programs.h"
+#include "padthv1_tuning.h"
 
 #include "padthv1_sched.h"
 
@@ -55,8 +56,8 @@
 //    Copyright (C) 2007 jorgen, linux-vst.com
 //
 
-const uint16_t MAX_VOICES = 32;			// polyphony
-const uint8_t  MAX_NOTES  = 128;
+const uint8_t MAX_VOICES  = 64;			// max polyphony
+const uint8_t MAX_NOTES   = 128;
 
 const float MIN_ENV_MSECS = 0.5f;		// min 500 usec per stage
 const float MAX_ENV_MSECS = 5000.0f;	// max 5 sec per stage (default)
@@ -68,6 +69,8 @@ const float TUNING_SCALE  = 1.0f;
 const float SWEEP_SCALE   = 0.5f;
 const float PITCH_SCALE   = 0.5f;
 
+const uint8_t MAX_DIRECT_NOTES = (MAX_VOICES >> 2);
+
 
 // maximum helper
 
@@ -137,9 +140,14 @@ inline float padthv1_pow2f ( const float x )
 
 // convert note to frequency (hertz)
 
-inline float padthv1_freq ( float note )
+inline float padthv1_freq2 ( float delta )
 {
-	return (440.0f / 32.0f) * ::powf(2.0f, (note - 9.0f) / 12.0f);
+	return ::powf(2.0f, delta / 12.0f);
+}
+
+inline float padthv1_freq ( int note )
+{
+	return (440.0f / 32.0f) * padthv1_freq2(float(note - 9));
 }
 
 
@@ -149,7 +157,7 @@ class padthv1_port
 {
 public:
 
-	padthv1_port() : m_port(NULL), m_value(0.0f), m_vport(0.0f) {}
+	padthv1_port() : m_port(nullptr), m_value(0.0f), m_vport(0.0f) {}
 
 	virtual ~padthv1_port() {}
 
@@ -193,6 +201,8 @@ public:
 
 	padthv1_port2() : m_vtick(0.0f), m_vstep(0.0f), m_nstep(0) {}
 
+	static const uint32_t NSTEP = 32;
+
 	void set_value(float value)
 	{
 		m_vtick = padthv1_port::value();
@@ -221,8 +231,6 @@ public:
 
 private:
 
-	static const uint32_t NSTEP = 32;
-
 	float    m_vtick;
 	float    m_vstep;
 	uint32_t m_nstep;
@@ -398,23 +406,6 @@ struct padthv1_ctl
 };
 
 
-// internal control
-
-struct padthv1_aux
-{
-	padthv1_aux() { reset(); }
-
-	void reset()
-	{
-		panning = 0.0f;
-		volume = 1.0f;
-	}
-
-	float panning;
-	float volume;
-};
-
-
 // dco
 
 struct padthv1_gen
@@ -453,6 +444,7 @@ struct padthv1_gen
 
 struct padthv1_dcf
 {
+	padthv1_port  enabled;
 	padthv1_port2 cutoff;
 	padthv1_port2 reso;
 	padthv1_port  type;
@@ -464,9 +456,11 @@ struct padthv1_dcf
 
 
 // lfo
+struct padthv1_voice;
 
 struct padthv1_lfo
 {
+	padthv1_port  enabled;
 	padthv1_port  shape;
 	padthv1_port  width;
 	padthv1_port2 bpm;
@@ -482,6 +476,8 @@ struct padthv1_lfo
 	padthv1_port2 volume;
 
 	padthv1_env   env;
+
+	padthv1_voice *psync;
 };
 
 
@@ -489,9 +485,9 @@ struct padthv1_lfo
 
 struct padthv1_dca
 {
-	padthv1_port2 volume;
+	padthv1_port volume;
 
-	padthv1_env   env;
+	padthv1_env  env;
 };
 
 
@@ -513,10 +509,10 @@ struct padthv1_def
 
 struct padthv1_out
 {
-	padthv1_port2 width;
-	padthv1_port2 panning;
-	padthv1_port2 fxsend;
-	padthv1_port2 volume;
+	padthv1_port width;
+	padthv1_port panning;
+	padthv1_port fxsend;
+	padthv1_port volume;
 };
 
 
@@ -524,11 +520,11 @@ struct padthv1_out
 
 struct padthv1_cho
 {
-	padthv1_port2 wet;
-	padthv1_port2 delay;
-	padthv1_port2 feedb;
-	padthv1_port2 rate;
-	padthv1_port2 mod;
+	padthv1_port wet;
+	padthv1_port delay;
+	padthv1_port feedb;
+	padthv1_port rate;
+	padthv1_port mod;
 };
 
 
@@ -536,10 +532,10 @@ struct padthv1_cho
 
 struct padthv1_fla
 {
-	padthv1_port2 wet;
-	padthv1_port2 delay;
-	padthv1_port2 feedb;
-	padthv1_port2 daft;
+	padthv1_port wet;
+	padthv1_port delay;
+	padthv1_port feedb;
+	padthv1_port daft;
 };
 
 
@@ -547,11 +543,11 @@ struct padthv1_fla
 
 struct padthv1_pha
 {
-	padthv1_port2 wet;
-	padthv1_port2 rate;
-	padthv1_port2 feedb;
-	padthv1_port2 depth;
-	padthv1_port2 daft;
+	padthv1_port wet;
+	padthv1_port rate;
+	padthv1_port feedb;
+	padthv1_port depth;
+	padthv1_port daft;
 };
 
 
@@ -559,10 +555,10 @@ struct padthv1_pha
 
 struct padthv1_del
 {
-	padthv1_port2 wet;
-	padthv1_port2 delay;
-	padthv1_port2 feedb;
-	padthv1_port2 bpm;
+	padthv1_port wet;
+	padthv1_port delay;
+	padthv1_port feedb;
+	padthv1_port bpm;
 };
 
 
@@ -570,13 +566,14 @@ struct padthv1_del
 
 struct padthv1_rev
 {
-	padthv1_port2 wet;
-	padthv1_port2 room;
-	padthv1_port2 damp;
-	padthv1_port2 feedb;
-	padthv1_port2 width;
+	padthv1_port wet;
+	padthv1_port room;
+	padthv1_port damp;
+	padthv1_port feedb;
+	padthv1_port width;
 };
 
+
 // dynamic(compressor/limiter)
 
 struct padthv1_dyn
@@ -586,6 +583,20 @@ struct padthv1_dyn
 };
 
 
+// keyboard/note range
+
+struct padthv1_key
+{
+	padthv1_port low;
+	padthv1_port high;
+
+	bool is_note(int key)
+	{
+		return (key >= int(*low) && int(*high) >= key);
+	}
+};
+
+
 // glide (portamento)
 
 struct padthv1_glide
@@ -650,26 +661,47 @@ protected:
 };
 
 
-// panning smoother (3 parameters)
+// balance smoother (1 parameters)
 
-class padthv1_pan : public padthv1_ramp3
+class padthv1_bal1 : public padthv1_ramp1
 {
 public:
 
-	padthv1_pan() : padthv1_ramp3(2) {}
+	padthv1_bal1() : padthv1_ramp1(2) {}
 
 protected:
 
 	float evaluate(uint16_t i)
 	{
-		padthv1_ramp3::update();
+		padthv1_ramp1::update();
 
-		const float wpan = 0.25f * M_PI
+		const float wbal = 0.25f * M_PI
+			* (1.0f + m_param1_v);
+
+		return M_SQRT2 * (i & 1 ? ::sinf(wbal) : ::cosf(wbal));
+	}
+};
+
+
+// balance smoother (2 parameters)
+
+class padthv1_bal2 : public padthv1_ramp2
+{
+public:
+
+	padthv1_bal2() : padthv1_ramp2(2) {}
+
+protected:
+
+	float evaluate(uint16_t i)
+	{
+		padthv1_ramp2::update();
+
+		const float wbal = 0.25f * M_PI
 			* (1.0f + m_param1_v)
-			* (1.0f + m_param2_v)
-			* (1.0f + m_param3_v);
+			* (1.0f + m_param2_v);
 
-		return M_SQRT2 * (i == 0 ? ::cosf(wpan) : ::sinf(wpan));
+		return M_SQRT2 * (i & 1 ? ::sinf(wbal) : ::cosf(wbal));
 	}
 };
 
@@ -761,6 +793,12 @@ struct padthv1_voice : public padthv1_list<padthv1_voice>
 
 	padthv1_pre dca1_pre;
 
+	float out1_panning;
+	float out1_volume;
+
+	padthv1_bal1  out1_pan;						// output panning
+	padthv1_ramp1 out1_vol;						// output volume
+
 	bool sustain;
 };
 
@@ -777,6 +815,8 @@ public:
 
 	void schedule_event()
 		{ if (m_enabled && ++m_count < 2) schedule(-1); }
+	void schedule_note(int key, int vel)
+		{ if (m_enabled) schedule((vel << 7) | key); }
 
 	void process(int) {}
 
@@ -797,6 +837,22 @@ private:
 };
 
 
+// micro-tuning/instance implementation
+
+class padthv1_tun
+{
+public:
+
+	padthv1_tun() : enabled(false), refPitch(440.0f), refNote(69) {}
+
+	bool    enabled;
+	float   refPitch;
+	int     refNote;
+	QString scaleFile;
+	QString keyMapFile;
+};
+
+
 // polyphonic synth implementation
 
 class padthv1_impl
@@ -828,11 +884,27 @@ public:
 	padthv1_controls *controls();
 	padthv1_programs *programs();
 
+	void setTuningEnabled(bool enabled);
+	bool isTuningEnabled() const;
+
+	void setTuningRefPitch(float refPitch);
+	float tuningRefPitch() const;
+
+	void setTuningRefNote(int refNote);
+	int tuningRefNote() const;
+
+	void setTuningScaleFile(const char *pszScaleFile);
+	const char *tuningScaleFile() const;
+
+	void setTuningKeyMapFile(const char *pszKeyMapFile);
+	const char *tuningKeyMapFile() const;
+
+	void resetTuning();
+
 	void process_midi(uint8_t *data, uint32_t size);
 	void process(float **ins, float **outs, uint32_t nframes);
 
-	bool sampleLoopTest();
-
+	void stabilize();
 	void reset();
 
 	void midiInEnabled(bool on);
@@ -840,6 +912,8 @@ public:
 
 	void directNoteOn(int note, int vel);
 
+	bool running(bool on);
+
 	padthv1_sample gen1_sample1, gen1_sample2;
 
 	padthv1_wave_lf lfo1_wave;
@@ -866,14 +940,19 @@ protected:
 		if (pv) {
 			m_free_list.remove(pv);
 			m_play_list.append(pv);
+			++m_nvoices;
 		}
 		return pv;
 	}
 
 	void free_voice ( padthv1_voice *pv )
 	{
+		if (m_lfo1.psync == pv)
+			m_lfo1.psync = nullptr;
+
 		m_play_list.remove(pv);
 		m_free_list.append(pv);
+		--m_nvoices;
 	}
 
 	void alloc_sfxs(uint32_t nsize);
@@ -884,11 +963,14 @@ private:
 	padthv1_controls m_controls;
 	padthv1_programs m_programs;
 	padthv1_midi_in  m_midi_in;
+	padthv1_tun      m_tun;
 
 	uint16_t m_nchannels;
 	float    m_srate;
 	float    m_bpm;
 
+	float    m_freqs[MAX_NOTES];
+
 	padthv1_ctl m_ctl1;
 
 	padthv1_gen m_gen1;
@@ -906,17 +988,17 @@ private:
 	padthv1_rev m_rev;
 	padthv1_dyn m_dyn;
 
+	padthv1_key m_key;
+
 	padthv1_voice **m_voices;
 	padthv1_voice  *m_notes[MAX_NOTES];
 
 	padthv1_list<padthv1_voice> m_free_list;
 	padthv1_list<padthv1_voice> m_play_list;
 
-	padthv1_aux   m_aux1;
-
 	padthv1_ramp1 m_wid1;
-	padthv1_pan   m_pan1;
-	padthv1_ramp4 m_vol1;
+	padthv1_bal2  m_pan1;
+	padthv1_ramp3 m_vol1;
 
 	float  **m_sfxs;
 	uint32_t m_nsize;
@@ -928,11 +1010,17 @@ private:
 	padthv1_fx_comp    *m_comp;
 
 	padthv1_reverb m_reverb;
-	padthv1_phasor m_phasor;
 
-	volatile int m_direct_chan;
-	volatile int m_direct_note;
-	volatile int m_direct_vel;
+	// process direct note on/off...
+	volatile uint16_t m_direct_note;
+
+	struct direct_note {
+		uint8_t status, note, vel;
+	} m_direct_notes[MAX_DIRECT_NOTES];
+
+	volatile int  m_nvoices;
+
+	volatile bool m_running;
 };
 
 
@@ -962,7 +1050,8 @@ padthv1_voice::padthv1_voice ( padthv1_impl *pImpl ) :
 padthv1_impl::padthv1_impl (
 	padthv1 *pSampl, uint16_t nchannels, float srate )
 	: gen1_sample1(pSampl, 1), gen1_sample2(pSampl, 2),
-		m_controls(pSampl), m_programs(pSampl), m_midi_in(pSampl), m_bpm(180.0f)
+		m_controls(pSampl), m_programs(pSampl),
+		m_midi_in(pSampl), m_bpm(180.0f), m_running(false)
 {
 	// null sample freqs.
 	m_gen1.sample1_0 = m_gen1.sample2_0 = 0.0f;
@@ -983,23 +1072,26 @@ padthv1_impl::padthv1_impl (
 	}
 
 	for (int note = 0; note < MAX_NOTES; ++note)
-		m_notes[note] = NULL;
+		m_notes[note] = nullptr;
 
 	// local buffers none yet
-	m_sfxs = NULL;
+	m_sfxs = nullptr;
 	m_nsize = 0;
 
 	// flangers none yet
-	m_flanger = NULL;
+	m_flanger = nullptr;
 
 	// phasers none yet
-	m_phaser = NULL;
+	m_phaser = nullptr;
 
 	// delays none yet
-	m_delay = NULL;
+	m_delay = nullptr;
 
 	// compressors none yet
-	m_comp = NULL;
+	m_comp = nullptr;
+
+	// Micro-tuning support, if any...
+	resetTuning();
 
 	// load controllers & programs database...
 	m_config.loadControls(&m_controls);
@@ -1014,6 +1106,8 @@ padthv1_impl::padthv1_impl (
 	// reset all voices
 	allControllersOff();
 	allNotesOff();
+
+	running(true);
 }
 
 
@@ -1048,25 +1142,25 @@ void padthv1_impl::setChannels ( uint16_t nchannels )
 	// deallocate flangers
 	if (m_flanger) {
 		delete [] m_flanger;
-		m_flanger = NULL;
+		m_flanger = nullptr;
 	}
 
 	// deallocate phasers
 	if (m_phaser) {
 		delete [] m_phaser;
-		m_phaser = NULL;
+		m_phaser = nullptr;
 	}
 
 	// deallocate delays
 	if (m_delay) {
 		delete [] m_delay;
-		m_delay = NULL;
+		m_delay = nullptr;
 	}
 
 	// deallocate compressors
 	if (m_comp) {
 		delete [] m_comp;
-		m_comp = NULL;
+		m_comp = nullptr;
 	}
 }
 
@@ -1132,7 +1226,7 @@ void padthv1_impl::alloc_sfxs ( uint32_t nsize )
 		for (uint16_t k = 0; k < m_nchannels; ++k)
 			delete [] m_sfxs[k];
 		delete [] m_sfxs;
-		m_sfxs = NULL;
+		m_sfxs = nullptr;
 		m_nsize = 0;
 	}
 
@@ -1180,7 +1274,7 @@ void padthv1_impl::setParamPort ( padthv1::ParamIndex index, float *pfParam )
 {
 	static float s_fDummy = 0.0f;
 
-	if (pfParam == NULL)
+	if (pfParam == nullptr)
 		pfParam = &s_fDummy;
 
 	padthv1_port *pParamPort = paramPort(index);
@@ -1198,8 +1292,7 @@ void padthv1_impl::setParamPort ( padthv1::ParamIndex index, float *pfParam )
 		m_vol1.reset(
 			m_out1.volume.value_ptr(),
 			m_dca1.volume.value_ptr(),
-			&m_ctl1.volume,
-			&m_aux1.volume);
+			&m_ctl1.volume);
 		break;
 	case padthv1::OUT1_WIDTH:
 		m_wid1.reset(
@@ -1208,8 +1301,7 @@ void padthv1_impl::setParamPort ( padthv1::ParamIndex index, float *pfParam )
 	case padthv1::OUT1_PANNING:
 		m_pan1.reset(
 			m_out1.panning.value_ptr(),
-			&m_ctl1.panning,
-			&m_aux1.panning);
+			&m_ctl1.panning);
 		break;
 	default:
 		break;
@@ -1219,7 +1311,7 @@ void padthv1_impl::setParamPort ( padthv1::ParamIndex index, float *pfParam )
 
 padthv1_port *padthv1_impl::paramPort ( padthv1::ParamIndex index )
 {
-	padthv1_port *pParamPort = NULL;
+	padthv1_port *pParamPort = nullptr;
 
 	switch (index) {
 	case padthv1::GEN1_SAMPLE1:   pParamPort = &m_gen1.sample1;     break;
@@ -1242,6 +1334,7 @@ padthv1_port *padthv1_impl::paramPort ( padthv1::ParamIndex index )
 	case padthv1::GEN1_OCTAVE:    pParamPort = &m_gen1.octave;      break;
 	case padthv1::GEN1_TUNING:    pParamPort = &m_gen1.tuning;      break;
 	case padthv1::GEN1_ENVTIME:   pParamPort = &m_gen1.envtime;     break;
+	case padthv1::DCF1_ENABLED:   pParamPort = &m_dcf1.enabled;     break;
 	case padthv1::DCF1_CUTOFF:    pParamPort = &m_dcf1.cutoff;      break;
 	case padthv1::DCF1_RESO:      pParamPort = &m_dcf1.reso;        break;
 	case padthv1::DCF1_TYPE:      pParamPort = &m_dcf1.type;        break;
@@ -1251,6 +1344,7 @@ padthv1_port *padthv1_impl::paramPort ( padthv1::ParamIndex index )
 	case padthv1::DCF1_DECAY:     pParamPort = &m_dcf1.env.decay;   break;
 	case padthv1::DCF1_SUSTAIN:   pParamPort = &m_dcf1.env.sustain; break;
 	case padthv1::DCF1_RELEASE:   pParamPort = &m_dcf1.env.release; break;
+	case padthv1::LFO1_ENABLED:   pParamPort = &m_lfo1.enabled;     break;
 	case padthv1::LFO1_SHAPE:     pParamPort = &m_lfo1.shape;       break;
 	case padthv1::LFO1_WIDTH:     pParamPort = &m_lfo1.width;       break;
 	case padthv1::LFO1_BPM:       pParamPort = &m_lfo1.bpm;         break;
@@ -1308,6 +1402,8 @@ padthv1_port *padthv1_impl::paramPort ( padthv1::ParamIndex index )
 	case padthv1::REV1_WIDTH:     pParamPort = &m_rev.width;        break;
 	case padthv1::DYN1_COMPRESS:  pParamPort = &m_dyn.compress;     break;
 	case padthv1::DYN1_LIMITER:   pParamPort = &m_dyn.limiter;      break;
+	case padthv1::KEY1_LOW:       pParamPort = &m_key.low;          break;
+	case padthv1::KEY1_HIGH:      pParamPort = &m_key.high;         break;
 	default: break;
 	}
 
@@ -1370,14 +1466,20 @@ void padthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 		if (++i >= size)
 			break;
 
-		// channel filter
-		if (!on)
-			continue;
-
+		// channel value
 		const int value = (data[i] & 0x7f);
 
+		// channel/controller filter
+		if (!on) {
+			if (status == 0xb0)
+				m_controls.process_enqueue(channel, key, value);
+			continue;
+		}
+
 		// note on
 		if (status == 0x90 && value > 0) {
+			if (!m_key.is_note(key))
+				continue;
 			padthv1_voice *pv;
 			// mono voice modes
 			if (*m_def.mono > 0.0f) {
@@ -1389,7 +1491,7 @@ void padthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 						m_lfo1.env.note_off_fast(&pv->lfo1_env);
 						m_dca1.env.note_off_fast(&pv->dca1_env);
 						if (++n > 1) { // there shall be only one
-							m_notes[pv->note] = NULL;
+							m_notes[pv->note] = nullptr;
 							pv->note = -1;
 						}
 					}
@@ -1401,7 +1503,7 @@ void padthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 				m_dcf1.env.note_off_fast(&pv->dcf1_env);
 				m_lfo1.env.note_off_fast(&pv->lfo1_env);
 				m_dca1.env.note_off_fast(&pv->dca1_env);
-				m_notes[pv->note] = NULL;
+				m_notes[pv->note] = nullptr;
 				pv->note = -1;
 			}
 			// find free voice
@@ -1422,53 +1524,66 @@ void padthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 					m_def.pressure.value_ptr(),
 					&m_ctl1.pressure, &pv->pre);
 				// frequencies
-				const float note1 = float(key)
-					+ *m_gen1.octave * OCTAVE_SCALE
+				const float gen1_tuning
+					= *m_gen1.octave * OCTAVE_SCALE
 					+ *m_gen1.tuning * TUNING_SCALE;
-				const float detune1	= *m_gen1.detune1 * DETUNE_SCALE;
-				const float detune2	= *m_gen1.detune2 * DETUNE_SCALE;
-				pv->gen1_freq1 = padthv1_freq(note1 + detune1);
-				pv->gen1_freq2 = padthv1_freq(note1 + detune2);
+				const float gen1_detune1
+					= *m_gen1.detune1 * DETUNE_SCALE;
+				const float gen1_detune2
+					= *m_gen1.detune2 * DETUNE_SCALE;
+				pv->gen1_freq1 = m_freqs[key] * padthv1_freq2(gen1_tuning + gen1_detune1);
+				pv->gen1_freq2 = m_freqs[key] * padthv1_freq2(gen1_tuning + gen1_detune2);
 				// phases
-				const float phase1 = *m_gen1.phase * PHASE_SCALE;
-				pv->gen1_sample1 = pv->gen1_osc1.start(  0.0f, pv->gen1_freq1);
-				pv->gen1_sample2 = pv->gen1_osc2.start(phase1, pv->gen1_freq2);
+				const float gen1_phase = *m_gen1.phase * PHASE_SCALE;
+				pv->gen1_sample1 = pv->gen1_osc1.start(      0.0f, pv->gen1_freq1);
+				pv->gen1_sample2 = pv->gen1_osc2.start(gen1_phase, pv->gen1_freq2);
 				// filters
-				const int type1 = int(*m_dcf1.type);
-				pv->dcf11.reset(padthv1_filter1::Type(type1));
-				pv->dcf12.reset(padthv1_filter1::Type(type1));
-				pv->dcf13.reset(padthv1_filter2::Type(type1));
-				pv->dcf14.reset(padthv1_filter2::Type(type1));
-				pv->dcf15.reset(padthv1_filter3::Type(type1));
-				pv->dcf16.reset(padthv1_filter3::Type(type1));
+				const int dcf1_type = int(*m_dcf1.type);
+				pv->dcf11.reset(padthv1_filter1::Type(dcf1_type));
+				pv->dcf12.reset(padthv1_filter1::Type(dcf1_type));
+				pv->dcf13.reset(padthv1_filter2::Type(dcf1_type));
+				pv->dcf14.reset(padthv1_filter2::Type(dcf1_type));
+				pv->dcf15.reset(padthv1_filter3::Type(dcf1_type));
+				pv->dcf16.reset(padthv1_filter3::Type(dcf1_type));
 				// formant filters
-				const float cutoff1 = *m_dcf1.cutoff;
-				const float reso1 = *m_dcf1.reso;
-				pv->dcf17.reset_filters(cutoff1, reso1);
-				pv->dcf18.reset_filters(cutoff1, reso1);
+				const float dcf1_cutoff = *m_dcf1.cutoff;
+				const float dcf1_reso = *m_dcf1.reso;
+				pv->dcf17.reset_filters(dcf1_cutoff, dcf1_reso);
+				pv->dcf18.reset_filters(dcf1_cutoff, dcf1_reso);
 				// envelopes
 				m_dcf1.env.start(&pv->dcf1_env);
 				m_lfo1.env.start(&pv->lfo1_env);
 				m_dca1.env.start(&pv->dca1_env);
 				// lfos
-				const float pshift1
-					= (*m_lfo1.sync > 0.0f ? m_phasor.pshift() : 0.0f);
-				pv->lfo1_sample = pv->lfo1.start(pshift1);
+				const float lfo1_pshift
+					= (m_lfo1.psync ? m_lfo1.psync->lfo1.pshift() : 0.0f);
+				pv->lfo1_sample = pv->lfo1.start(lfo1_pshift);
+				if (*m_lfo1.sync > 0.0f && m_lfo1.psync == nullptr)
+					m_lfo1.psync = pv;
 				// glides (portamentoa)
-				const float frames1
+				const float gen1_frames1
 					= uint32_t(*m_gen1.glide1 * *m_gen1.glide1 * m_srate);
-				const float frames2
+				const float gen1_frames2
 					= uint32_t(*m_gen1.glide2 * *m_gen1.glide2 * m_srate);
-				pv->gen1_glide1.reset(frames1, pv->gen1_freq1);
-				pv->gen1_glide2.reset(frames2, pv->gen1_freq2);
+				pv->gen1_glide1.reset(gen1_frames1, pv->gen1_freq1);
+				pv->gen1_glide2.reset(gen1_frames2, pv->gen1_freq2);
+				// panning
+				pv->out1_panning = 0.0f;
+				pv->out1_pan.reset(&pv->out1_panning);
+				// volume
+				pv->out1_volume = 0.0f;
+				pv->out1_vol.reset(&pv->out1_volume);
 				// sustain
 				pv->sustain = false;
 				// allocated
 				m_notes[key] = pv;
 			}
+			m_midi_in.schedule_note(key, value);
 		}
 		// note off
 		else if (status == 0x80 || (status == 0x90 && value == 0)) {
+			if (!m_key.is_note(key))
+				continue;
 			padthv1_voice *pv = m_notes[key];
 			if (pv && pv->note >= 0) {
 				if (m_ctl1.sustain)
@@ -1479,7 +1594,7 @@ void padthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 					m_dcf1.env.note_off(&pv->dcf1_env);
 					m_lfo1.env.note_off(&pv->lfo1_env);
 				}
-				m_notes[pv->note] = NULL;
+				m_notes[pv->note] = nullptr;
 				pv->note = -1;
 				// mono legato?
 				if (*m_def.mono > 0.0f) {
@@ -1493,9 +1608,12 @@ void padthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 					}
 				}
 			}
+			m_midi_in.schedule_note(key, 0);
 		}
 		// key pressure/poly.aftertouch
 		else if (status == 0xa0) {
+			if (!m_key.is_note(key))
+				continue;
 			padthv1_voice *pv = m_notes[key];
 			if (pv && pv->note >= 0)
 				pv->pre = *m_def.pressure * float(value) / 127.0f;
@@ -1550,7 +1668,6 @@ void padthv1_impl::process_midi ( uint8_t *data, uint32_t size )
 			const float pitchbend = float(key + (value << 7) - 0x2000) / 8192.0f;
 			m_ctl1.pitchbend = padthv1_pow2f(*m_def.pitchbend * pitchbend);
 		}
-
 	}
 
 	// process pending controllers...
@@ -1598,16 +1715,16 @@ void padthv1_impl::allNotesOff (void)
 	padthv1_voice *pv = m_play_list.next();
 	while (pv) {
 		if (pv->note >= 0)
-			m_notes[pv->note] = NULL;
+			m_notes[pv->note] = nullptr;
 		free_voice(pv);
 		pv = m_play_list.next();
 	}
 
 	gen1_last1 = gen1_last2 = 0.0f;
 
-	m_aux1.reset();
+	m_lfo1.psync = nullptr;
 
-	m_direct_chan = m_direct_note = m_direct_vel = -1;
+	m_direct_note = 0;
 }
 
 
@@ -1623,7 +1740,7 @@ void padthv1_impl::allSustainOff (void)
 				m_dca1.env.note_off(&pv->dca1_env);
 				m_dcf1.env.note_off(&pv->dcf1_env);
 				m_lfo1.env.note_off(&pv->lfo1_env);
-				m_notes[pv->note] = NULL;
+				m_notes[pv->note] = nullptr;
 				pv->note = -1;
 			}
 		}
@@ -1635,13 +1752,18 @@ void padthv1_impl::allSustainOff (void)
 // direct note-on triggered on next cycle...
 void padthv1_impl::directNoteOn ( int note, int vel )
 {
-	if (vel > 0) {
+	if (vel > 0 && m_nvoices >= MAX_DIRECT_NOTES)
+		return;
+
+	const uint32_t i = m_direct_note;
+	if (i < MAX_DIRECT_NOTES) {
 		const int ch1 = int(*m_def.channel);
-		m_direct_chan = (ch1 > 0 ? ch1 - 1 : 0) & 0x0f;
-		m_direct_note = note;
-		m_direct_vel  = vel;
-	} else {
-		m_direct_vel  = 0;
+		const int chan = (ch1 > 0 ? ch1 - 1 : 0) & 0x0f;
+		direct_note& data = m_direct_notes[i];
+		data.status = (vel > 0 ? 0x90 : 0x80) | chan;
+		data.note = note;
+		data.vel = vel;
+		++m_direct_note;
 	}
 }
 
@@ -1662,6 +1784,113 @@ padthv1_programs *padthv1_impl::programs (void)
 }
 
 
+// Micro-tuning support
+
+void padthv1_impl::setTuningEnabled ( bool enabled )
+{
+	m_tun.enabled = enabled;
+}
+
+bool padthv1_impl::isTuningEnabled (void) const
+{
+	return m_tun.enabled;
+}
+
+
+void padthv1_impl::setTuningRefPitch ( float refPitch )
+{
+	m_tun.refPitch = refPitch;
+}
+
+float padthv1_impl::tuningRefPitch (void) const
+{
+	return m_tun.refPitch;
+}
+
+
+void padthv1_impl::setTuningRefNote ( int refNote )
+{
+	m_tun.refNote = refNote;
+}
+
+int padthv1_impl::tuningRefNote (void) const
+{
+	return m_tun.refNote;
+}
+
+
+void padthv1_impl::setTuningScaleFile ( const char *pszScaleFile )
+{
+	m_tun.scaleFile = QString::fromUtf8(pszScaleFile);
+}
+
+const char *padthv1_impl::tuningScaleFile (void) const
+{
+	return m_tun.scaleFile.toUtf8().constData();
+}
+
+
+void padthv1_impl::setTuningKeyMapFile ( const char *pszKeyMapFile )
+{
+	m_tun.keyMapFile = QString::fromUtf8(pszKeyMapFile);
+}
+
+const char *padthv1_impl::tuningKeyMapFile (void) const
+{
+	return m_tun.keyMapFile.toUtf8().constData();
+}
+
+
+void padthv1_impl::resetTuning (void)
+{
+	if (m_tun.enabled) {
+		// Instance micro-tuning, possibly from Scala keymap and scale files...
+		padthv1_tuning tuning(
+			m_tun.refPitch,
+			m_tun.refNote);
+		if (m_tun.keyMapFile.isEmpty())
+		if (!m_tun.keyMapFile.isEmpty())
+			tuning.loadKeyMapFile(m_tun.keyMapFile);
+		if (!m_tun.scaleFile.isEmpty())
+			tuning.loadScaleFile(m_tun.scaleFile);
+		for (int note = 0; note < MAX_NOTES; ++note)
+			m_freqs[note] = tuning.noteToPitch(note);
+		// Done instance tuning.
+	}
+	else
+	if (m_config.bTuningEnabled) {
+		// Global/config micro-tuning, possibly from Scala keymap and scale files...
+		padthv1_tuning tuning(
+			m_config.fTuningRefPitch,
+			m_config.iTuningRefNote);
+		if (!m_config.sTuningKeyMapFile.isEmpty())
+			tuning.loadKeyMapFile(m_config.sTuningKeyMapFile);
+		if (!m_config.sTuningScaleFile.isEmpty())
+			tuning.loadScaleFile(m_config.sTuningScaleFile);
+		for (int note = 0; note < MAX_NOTES; ++note)
+			m_freqs[note] = tuning.noteToPitch(note);
+		// Done global/config tuning.
+	} else {
+		// Native/default tuning, 12-tone equal temperament western standard...
+		for (int note = 0; note < MAX_NOTES; ++note)
+			m_freqs[note] = padthv1_freq(note);
+		// Done native/default tuning.
+	}
+}
+
+
+// all stabilize
+
+void padthv1_impl::stabilize (void)
+{
+	for (int i = 0; i < padthv1::NUM_PARAMS; ++i) {
+		padthv1_port *pParamPort = paramPort(padthv1::ParamIndex(i));
+		if (pParamPort)
+			pParamPort->tick(padthv1_port2::NSTEP);
+	}
+}
+
+
 // all reset clear
 
 void padthv1_impl::reset (void)
@@ -1669,29 +1898,27 @@ void padthv1_impl::reset (void)
 	m_vol1.reset(
 		m_out1.volume.value_ptr(),
 		m_dca1.volume.value_ptr(),
-		&m_ctl1.volume,
-		&m_aux1.volume);
+		&m_ctl1.volume);
 	m_pan1.reset(
 		m_out1.panning.value_ptr(),
-		&m_ctl1.panning,
-		&m_aux1.panning);
+		&m_ctl1.panning);
 	m_wid1.reset(
 		m_out1.width.value_ptr());
 
 	// flangers
-	if (m_flanger == NULL)
+	if (m_flanger == nullptr)
 		m_flanger = new padthv1_fx_flanger [m_nchannels];
 
 	// phasers
-	if (m_phaser == NULL)
+	if (m_phaser == nullptr)
 		m_phaser = new padthv1_fx_phaser [m_nchannels];
 
 	// delays
-	if (m_delay == NULL)
+	if (m_delay == nullptr)
 		m_delay = new padthv1_fx_delay [m_nchannels];
 
 	// compressors
-	if (m_comp == NULL)
+	if (m_comp == nullptr)
 		m_comp = new padthv1_fx_comp [m_nchannels];
 
 	// reverbs
@@ -1724,6 +1951,8 @@ uint32_t padthv1_impl::midiInCount (void)
 
 void padthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 {
+	if (!m_running) return;
+
 	float *v_outs[m_nchannels];
 	float *v_sfxs[m_nchannels];
 
@@ -1733,29 +1962,29 @@ void padthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 	uint16_t k;
 
 	for (k = 0; k < m_nchannels; ++k) {
-		::memcpy(m_sfxs[k], ins[k], nframes * sizeof(float));
-		::memset(outs[k], 0, nframes * sizeof(float));
+		::memset(m_sfxs[k], 0, nframes * sizeof(float));
+		::memcpy(outs[k], ins[k], nframes * sizeof(float));
 	}
 
 	// process direct note on/off...
-	if (m_direct_chan >= 0 && m_direct_note >= 0 && m_direct_vel >= 0) {
-		struct note_data { uint8_t status, note, vel; } data;
-		data.status = (m_direct_vel > 0 ? 0x90 : 0x80) | m_direct_chan;
-		data.note = m_direct_note;
-		data.vel = m_direct_vel;
+	while (m_direct_note > 0) {
+		const direct_note& data
+			= m_direct_notes[--m_direct_note];
 		process_midi((uint8_t *) &data, sizeof(data));
-		if (m_direct_vel == 0) {
-			m_direct_chan = -1;
-			m_direct_note = -1;
-		}
-		m_direct_vel = -1;
 	}
 
 	// controls
-	const float lfo1_freq
-		= get_bpm(*m_lfo1.bpm) / (60.01f - *m_lfo1.rate * 60.0f);
 
-	const float modwheel1 = m_ctl1.modwheel + PITCH_SCALE * *m_lfo1.pitch;
+	const bool lfo1_enabled = (*m_lfo1.enabled > 0.0f);
+	
+	const float lfo1_freq = (lfo1_enabled
+		? get_bpm(*m_lfo1.bpm) / (60.01f - *m_lfo1.rate * 60.0f) : 0.0f);
+
+	const float modwheel1 = (lfo1_enabled
+		? m_ctl1.modwheel + PITCH_SCALE * *m_lfo1.pitch : 0.0f);
+
+	const bool dcf1_enabled = (*m_dcf1.enabled > 0.0f);
+	
 	const float fxsend1 = *m_out1.fxsend * *m_out1.fxsend;
 
 	if (m_gen1.sample1_0 != *m_gen1.sample1) {
@@ -1780,8 +2009,10 @@ void padthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 		updateEnvTimes();
 	}
 
-	lfo1_wave.reset_test(
-		padthv1_wave::Shape(*m_lfo1.shape), *m_lfo1.width);
+	if (lfo1_enabled) {
+		lfo1_wave.reset_test(
+			padthv1_wave::Shape(*m_lfo1.shape), *m_lfo1.width);
+	}
 
 	// per voice
 
@@ -1822,8 +2053,10 @@ void padthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 				// generators
 
-				const float lfo1_env = pv->lfo1_env.tick();
-				const float lfo1 = pv->lfo1_sample * lfo1_env;
+				const float lfo1_env
+					= (lfo1_enabled ? pv->lfo1_env.tick() : 0.0f);
+				const float lfo1
+					= (lfo1_enabled ? pv->lfo1_sample * lfo1_env : 0.0f);
 
 				const float gen1 = pv->gen1_sample1 * pv->gen1_bal.value(j, 0);
 				const float gen2 = pv->gen1_sample2 * pv->gen1_bal.value(j, 1);
@@ -1835,8 +2068,10 @@ void padthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 					* (m_ctl1.pitchbend + modwheel1 * lfo1)
 					+ pv->gen1_glide2.tick());
 
-				pv->lfo1_sample = pv->lfo1.sample(lfo1_freq
-					* (1.0f + SWEEP_SCALE * *m_lfo1.sweep * lfo1_env));
+				if (lfo1_enabled) {
+					pv->lfo1_sample = pv->lfo1.sample(lfo1_freq
+						* (1.0f + SWEEP_SCALE * *m_lfo1.sweep * lfo1_env));
+				}
 
 				// ring modulators
 
@@ -1848,31 +2083,32 @@ void padthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 				// filters
 
-				const float env1 = 0.5f * (1.0f + vel1
-					* *m_dcf1.envelope * pv->dcf1_env.tick());
-				const float cutoff1 = padthv1_sigmoid_1(*m_dcf1.cutoff
-					* env1 * (1.0f + *m_lfo1.cutoff * lfo1));
-				const float reso1 = padthv1_sigmoid_1(*m_dcf1.reso
-					* env1 * (1.0f + *m_lfo1.reso * lfo1));
-
-				switch (int(*m_dcf1.slope)) {
-				case 3: // Formant
-					mod1 = pv->dcf17.output(mod1, cutoff1, reso1);
-					mod2 = pv->dcf18.output(mod2, cutoff1, reso1);
-					break;
-				case 2: // Biquad
-					mod1 = pv->dcf15.output(mod1, cutoff1, reso1);
-					mod2 = pv->dcf16.output(mod2, cutoff1, reso1);
-					break;
-				case 1: // 24db/octave
-					mod1 = pv->dcf13.output(mod1, cutoff1, reso1);
-					mod2 = pv->dcf14.output(mod2, cutoff1, reso1);
-					break;
-				case 0: // 12db/octave
-				default:
-					mod1 = pv->dcf11.output(mod1, cutoff1, reso1);
-					mod2 = pv->dcf12.output(mod2, cutoff1, reso1);
-					break;
+				if (dcf1_enabled) {
+					const float env1 = 0.5f * (1.0f + vel1
+						* *m_dcf1.envelope * pv->dcf1_env.tick());
+					const float cutoff1 = padthv1_sigmoid_1(*m_dcf1.cutoff
+						* env1 * (1.0f + *m_lfo1.cutoff * lfo1));
+					const float reso1 = padthv1_sigmoid_1(*m_dcf1.reso
+						* env1 * (1.0f + *m_lfo1.reso * lfo1));
+					switch (int(*m_dcf1.slope)) {
+					case 3: // Formant
+						mod1 = pv->dcf17.output(mod1, cutoff1, reso1);
+						mod2 = pv->dcf18.output(mod2, cutoff1, reso1);
+						break;
+					case 2: // Biquad
+						mod1 = pv->dcf15.output(mod1, cutoff1, reso1);
+						mod2 = pv->dcf16.output(mod2, cutoff1, reso1);
+						break;
+					case 1: // 24db/octave
+						mod1 = pv->dcf13.output(mod1, cutoff1, reso1);
+						mod2 = pv->dcf14.output(mod2, cutoff1, reso1);
+						break;
+					case 0: // 12db/octave
+					default:
+						mod1 = pv->dcf11.output(mod1, cutoff1, reso1);
+						mod2 = pv->dcf12.output(mod2, cutoff1, reso1);
+						break;
+					}
 				}
 
 				// volumes
@@ -1881,14 +2117,17 @@ void padthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 				const float mid1 = 0.5f * (mod1 + mod2);
 				const float sid1 = 0.5f * (mod1 - mod2);
 				const float vol1 = vel1 * m_vol1.value(j)
-					* pv->dca1_env.tick();
+					* pv->dca1_env.tick()
+					* pv->out1_vol.value(j);
 
 				// outputs
 
-				const float out1
-					= vol1 * (mid1 + sid1 * wid1) * m_pan1.value(j, 0);
-				const float out2
-					= vol1 * (mid1 - sid1 * wid1) * m_pan1.value(j, 1);
+				const float out1 = vol1 * (mid1 + sid1 * wid1)
+					* pv->out1_pan.value(j, 0)
+					* m_pan1.value(j, 0);
+				const float out2 = vol1 * (mid1 - sid1 * wid1)
+					* pv->out1_pan.value(j, 1)
+					* m_pan1.value(j, 1);
 
 				for (k = 0; k < m_nchannels; ++k) {
 					const float dry = (k & 1 ? out2 : out1);
@@ -1899,8 +2138,8 @@ void padthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 				if (j == 0) {
 					pv->gen1_balance = lfo1 * *m_lfo1.balance;
-					m_aux1.panning = lfo1 * *m_lfo1.panning;
-					m_aux1.volume  = lfo1 * *m_lfo1.volume + 1.0f;
+					pv->out1_panning = lfo1 * *m_lfo1.panning;
+					pv->out1_volume  = lfo1 * *m_lfo1.volume + 1.0f;
 				}
 			}
 
@@ -1910,6 +2149,8 @@ void padthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 
 			pv->gen1_bal.process(ngen);
 			pv->dca1_pre.process(ngen);
+			pv->out1_pan.process(ngen);
+			pv->out1_vol.process(ngen);
 
 			// envelope countdowns
 
@@ -1962,27 +2203,24 @@ void padthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 	// output mix-down
 	for (k = 0; k < m_nchannels; ++k) {
 		uint32_t n;
-		// fx sends
-		float *in  = m_sfxs[k];
-		float *out = outs[k];
-		for (n = 0; n < nframes; ++n)
-			*out++ += *in++;
-		// dynamics
-		in = outs[k];
+		float *sfx = m_sfxs[k];
 		// compressor
 		if (int(*m_dyn.compress) > 0)
-			m_comp[k].process(in, nframes);
+			m_comp[k].process(sfx, nframes);
 		// limiter
 		if (int(*m_dyn.limiter) > 0) {
-			out = in;
+			float *p = sfx;
+			float *q = sfx;
 			for (n = 0; n < nframes; ++n)
-				*out++ = padthv1_sigmoid(*in++);
+				*q++ = padthv1_sigmoid(*p++);
 		}
+		// mix-down
+		float *out = outs[k];
+		for (n = 0; n < nframes; ++n)
+			*out++ += *sfx++;
 	}
 
 	// post-processing
-	m_phasor.process(nframes);
-
 	m_dca1.volume.tick(nframes);
 	m_out1.width.tick(nframes);
 	m_out1.panning.tick(nframes);
@@ -1996,6 +2234,15 @@ void padthv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 }
 
 
+// process running state...
+bool padthv1_impl::running ( bool on )
+{
+	const bool running = m_running;
+	m_running = on;
+	return running;
+}
+
+
 //-------------------------------------------------------------------------
 // padthv1 - decl.
 //
@@ -2044,7 +2291,7 @@ padthv1_sample *padthv1::sample ( int sid ) const
 	if (sid == 2)
 		return &(m_pImpl->gen1_sample2);
 	else
-		return NULL;
+		return nullptr;
 }
 
 
@@ -2129,6 +2376,22 @@ padthv1_programs *padthv1::programs (void) const
 }
 
 
+// process state
+
+bool padthv1::running ( bool on )
+{
+	return m_pImpl->running(on);
+}
+
+
+// all stabilize
+
+void padthv1::stabilize (void)
+{
+	m_pImpl->stabilize();
+}
+
+
 // all reset clear
 
 void padthv1::reset (void)
@@ -2159,4 +2422,66 @@ void padthv1::directNoteOn ( int note, int vel )
 }
 
 
+// Micro-tuning support
+void padthv1::setTuningEnabled ( bool enabled )
+{
+	m_pImpl->setTuningEnabled(enabled);
+}
+
+bool padthv1::isTuningEnabled (void) const
+{
+	return m_pImpl->isTuningEnabled();
+}
+
+
+void padthv1::setTuningRefPitch ( float refPitch )
+{
+	m_pImpl->setTuningRefPitch(refPitch);
+}
+
+float padthv1::tuningRefPitch (void) const
+{
+	return m_pImpl->tuningRefPitch();
+}
+
+
+void padthv1::setTuningRefNote ( int refNote )
+{
+	m_pImpl->setTuningRefNote(refNote);
+}
+
+int padthv1::tuningRefNote (void) const
+{
+	return m_pImpl->tuningRefNote();
+}
+
+
+void padthv1::setTuningScaleFile ( const char *pszScaleFile )
+{
+	m_pImpl->setTuningScaleFile(pszScaleFile);
+}
+
+const char *padthv1::tuningScaleFile (void) const
+{
+	return m_pImpl->tuningScaleFile();
+}
+
+
+void padthv1::setTuningKeyMapFile ( const char *pszKeyMapFile )
+{
+	m_pImpl->setTuningKeyMapFile(pszKeyMapFile);
+}
+
+const char *padthv1::tuningKeyMapFile (void) const
+{
+	return m_pImpl->tuningKeyMapFile();
+}
+
+
+void padthv1::resetTuning (void)
+{
+	m_pImpl->resetTuning();
+}
+
+
 // end of padthv1.cpp
