@@ -1,7 +1,7 @@
 // padthv1.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -82,6 +82,7 @@ public:
 		GEN1_OCTAVE,
 		GEN1_TUNING,
 		GEN1_ENVTIME,
+		DCF1_ENABLED,
 		DCF1_CUTOFF,
 		DCF1_RESO,
 		DCF1_TYPE,
@@ -91,6 +92,7 @@ public:
 		DCF1_DECAY,
 		DCF1_SUSTAIN,
 		DCF1_RELEASE,
+		LFO1_ENABLED,
 		LFO1_SHAPE,
 		LFO1_WIDTH,
 		LFO1_BPM,
@@ -151,6 +153,9 @@ public:
 		DYN1_COMPRESS,
 		DYN1_LIMITER,
 
+		KEY1_LOW,
+		KEY1_HIGH,
+
 		NUM_PARAMS
 	};
 
@@ -160,6 +165,9 @@ public:
 	void setParamValue(ParamIndex index, float fValue);
 	float paramValue(ParamIndex index) const;
 
+	bool running(bool on);
+
+	void stabilize();
 	void reset();
 
 	padthv1_controls *controls() const;
@@ -175,6 +183,25 @@ public:
 
 	void directNoteOn(int note, int vel);
 
+	void setTuningEnabled(bool enabled);
+	bool isTuningEnabled() const;
+
+	void setTuningRefPitch(float refPitch);
+	float tuningRefPitch() const;
+
+	void setTuningRefNote(int refNote);
+	int tuningRefNote() const;
+
+	void setTuningScaleFile(const char *pszScaleFile);
+	const char *tuningScaleFile() const;
+
+	void setTuningKeyMapFile(const char *pszKeyMapFile);
+	const char *tuningKeyMapFile() const;
+
+	void resetTuning();
+
+	virtual void updateTuning() = 0;
+
 private:
 
 	padthv1_impl *m_pImpl;
