@@ -1,7 +1,7 @@
 // padthv1_lv2.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -35,6 +35,10 @@
 #include "lv2/lv2plug.in/ns/ext/options/options.h"
 #include "lv2/lv2plug.in/ns/ext/buf-size/buf-size.h"
 
+#ifdef CONFIG_LV2_PATCH
+#include "lv2/lv2plug.in/ns/ext/patch/patch.h"
+#endif
+
 #ifndef CONFIG_LV2_ATOM_FORGE_OBJECT
 #define lv2_atom_forge_object(forge, frame, id, otype) \
 		lv2_atom_forge_blank(forge, frame, id, otype)
@@ -52,6 +56,7 @@
 #include <stdlib.h>
 #include <math.h>
 
+#include <QApplication>
 #include <QDomDocument>
 
 
@@ -71,27 +76,31 @@ padthv1_lv2::padthv1_lv2 (
 {
 	::memset(&m_urids, 0, sizeof(m_urids));
 
-	m_urid_map = NULL;
-	m_atom_in  = NULL;
-	m_atom_out = NULL;
-	m_schedule = NULL;
+	m_urid_map = nullptr;
+	m_atom_in  = nullptr;
+	m_atom_out = nullptr;
+	m_schedule = nullptr;
 	m_ndelta   = 0;
 
-	const LV2_Options_Option *host_options = NULL;
+	const LV2_Options_Option *host_options = nullptr;
 
 	for (int i = 0; host_features && host_features[i]; ++i) {
 		const LV2_Feature *host_feature = host_features[i];
 		if (::strcmp(host_feature->URI, LV2_URID_MAP_URI) == 0) {
 			m_urid_map = (LV2_URID_Map *) host_feature->data;
 			if (m_urid_map) {
- 				m_urids.gen1_sample = m_urid_map->map(
- 					m_urid_map->handle, PADTHV1_LV2_PREFIX "GEN1_SAMPLE");
- 				m_urids.gen1_loop_start = m_urid_map->map(
- 					m_urid_map->handle, PADTHV1_LV2_PREFIX "GEN1_LOOP_START");
- 				m_urids.gen1_loop_end = m_urid_map->map(
- 					m_urid_map->handle, PADTHV1_LV2_PREFIX "GEN1_LOOP_END");
-				m_urids.gen1_update = m_urid_map->map(
-					m_urid_map->handle, PADTHV1_LV2_PREFIX "GEN1_UPDATE");
+				m_urids.p201_tuning_enabled = m_urid_map->map(
+					m_urid_map->handle, PADTHV1_LV2_PREFIX "P201_TUNING_ENABLED");
+				m_urids.p202_tuning_refPitch = m_urid_map->map(
+					m_urid_map->handle, PADTHV1_LV2_PREFIX "P202_TUNING_REF_PITCH");
+				m_urids.p203_tuning_refNote = m_urid_map->map(
+					m_urid_map->handle, PADTHV1_LV2_PREFIX "P203_TUNING_REF_NOTE");
+				m_urids.p204_tuning_scaleFile = m_urid_map->map(
+					m_urid_map->handle, PADTHV1_LV2_PREFIX "P204_TUNING_SCALE_FILE");
+				m_urids.p205_tuning_keyMapFile = m_urid_map->map(
+					m_urid_map->handle, PADTHV1_LV2_PREFIX "P205_TUNING_KEYMAP_FILE");
+				m_urids.tun1_update = m_urid_map->map(
+					m_urid_map->handle, PADTHV1_LV2_PREFIX "TUN1_UPDATE");
 				m_urids.atom_Blank = m_urid_map->map(
 					m_urid_map->handle, LV2_ATOM__Blank);
 				m_urids.atom_Object = m_urid_map->map(
@@ -100,6 +109,8 @@ padthv1_lv2::padthv1_lv2 (
 					m_urid_map->handle, LV2_ATOM__Float);
 				m_urids.atom_Int = m_urid_map->map(
 					m_urid_map->handle, LV2_ATOM__Int);
+				m_urids.atom_Bool = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__Bool);
 				m_urids.atom_Path = m_urid_map->map(
 					m_urid_map->handle, LV2_ATOM__Path);
 				m_urids.time_Position = m_urid_map->map(
@@ -120,6 +131,20 @@ padthv1_lv2::padthv1_lv2 (
 			#endif
 				m_urids.state_StateChanged = m_urid_map->map(
 					m_urid_map->handle, LV2_STATE__StateChanged);
+			#ifdef CONFIG_LV2_PATCH
+				m_urids.patch_Get = m_urid_map->map(
+					m_urid_map->handle, LV2_PATCH__Get);
+				m_urids.patch_Set = m_urid_map->map(
+					m_urid_map->handle, LV2_PATCH__Set);
+				m_urids.patch_Put = m_urid_map->map(
+					m_urid_map->handle, LV2_PATCH__Put);
+				m_urids.patch_body = m_urid_map->map(
+					m_urid_map->handle, LV2_PATCH__body);
+				m_urids.patch_property = m_urid_map->map(
+					m_urid_map->handle, LV2_PATCH__property);
+				m_urids.patch_value = m_urid_map->map(
+ 					m_urid_map->handle, LV2_PATCH__value);
+			#endif
 			}
 		}
 		else
@@ -160,7 +185,7 @@ padthv1_lv2::padthv1_lv2 (
 	m_ins  = new float * [nchannels];
 	m_outs = new float * [nchannels];
 	for (uint16_t k = 0; k < nchannels; ++k)
-		m_ins[k] = m_outs[k] = NULL;
+		m_ins[k] = m_outs[k] = nullptr;
 }
 
 
@@ -218,7 +243,7 @@ void padthv1_lv2::run ( uint32_t nframes )
 
 	if (m_atom_in) {
 		LV2_ATOM_SEQUENCE_FOREACH(m_atom_in, event) {
-			if (event == NULL)
+			if (event == nullptr)
 				continue;
 			if (event->body.type == m_urids.midi_MidiEvent) {
 				uint8_t *data = (uint8_t *) LV2_ATOM_BODY(&event->body);
@@ -241,20 +266,79 @@ void padthv1_lv2::run ( uint32_t nframes )
 				const LV2_Atom_Object *object
 					= (LV2_Atom_Object *) &event->body;
 				if (object->body.otype == m_urids.time_Position) {
-					LV2_Atom *atom = NULL;
+					LV2_Atom *atom = nullptr;
 					lv2_atom_object_get(object,
-						m_urids.time_beatsPerMinute, &atom, NULL);
+						m_urids.time_beatsPerMinute, &atom, nullptr);
 					if (atom && atom->type == m_urids.atom_Float) {
 						const float host_bpm = ((LV2_Atom_Float *) atom)->body;
 						if (::fabsf(host_bpm - padthv1::tempo()) > 0.001f)
 							padthv1::setTempo(host_bpm);
 					}
 				}
+			#ifdef CONFIG_LV2_PATCH
+				else 
+				if (object->body.otype == m_urids.patch_Set) {
+					// set property value
+					const LV2_Atom *property = nullptr;
+					const LV2_Atom *value = nullptr;
+					lv2_atom_object_get(object,
+						m_urids.patch_property, &property,
+						m_urids.patch_value, &value, 0);
+					if (property && value && property->type == m_forge.URID) {
+						const uint32_t key = ((const LV2_Atom_URID *) property)->body;
+						const LV2_URID type = value->type;
+						if (key == m_urids.p201_tuning_enabled
+							&& type == m_urids.atom_Bool) {
+							const uint32_t enabled
+								= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
+							padthv1::setTuningEnabled(enabled > 0);
+							updateTuning();
+						}
+						else
+						if (key == m_urids.p202_tuning_refPitch
+							&& type == m_urids.atom_Float) {
+							const float refPitch
+								= *(float *) LV2_ATOM_BODY_CONST(value);
+							padthv1::setTuningRefPitch(refPitch);
+							updateTuning();
+						}
+						else
+						if (key == m_urids.p203_tuning_refNote
+							&& type == m_urids.atom_Int) {
+							const uint32_t refNote
+								= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
+							padthv1::setTuningRefNote(refNote);
+							updateTuning();
+						}
+						else
+						if (key == m_urids.p204_tuning_scaleFile
+							&& type == m_urids.atom_Path) {
+							const char *scaleFile
+								= (const char *) LV2_ATOM_BODY_CONST(value);
+							padthv1::setTuningScaleFile(scaleFile);
+							updateTuning();
+						}
+						else
+						if (key == m_urids.p205_tuning_keyMapFile
+							&& type == m_urids.atom_Path) {
+							const char *keyMapFile
+								= (const char *) LV2_ATOM_BODY_CONST(value);
+							padthv1::setTuningKeyMapFile(keyMapFile);
+							updateTuning();
+						}
+					}
+				}
+				else
+				if (object->body.otype == m_urids.patch_Get) {
+					// put all property values (probably to UI)
+					patch_put(ndelta);
+				}
+			#endif	// CONFIG_LV2_PATCH
 			}
 		}
 		// remember last time for worker response
 		m_ndelta = ndelta;
-	//	m_atom_in = NULL;
+	//	m_atom_in = nullptr;
 	}
 
 	if (nframes > ndelta)
@@ -280,6 +364,42 @@ uint32_t padthv1_lv2::urid_map ( const char *uri ) const
 }
 
 
+//-------------------------------------------------------------------------
+// padthv1_lv2 - Instantiation and cleanup.
+//
+
+QApplication *padthv1_lv2::g_qapp_instance = nullptr;
+unsigned int  padthv1_lv2::g_qapp_refcount = 0;
+
+
+void padthv1_lv2::qapp_instantiate (void)
+{
+	if (qApp == nullptr && g_qapp_instance == nullptr) {
+		static int s_argc = 1;
+		static const char *s_argv[] = { __func__, nullptr };
+		g_qapp_instance = new QApplication(s_argc, (char **) s_argv);
+	}
+
+	if (g_qapp_instance)
+		g_qapp_refcount++;
+}
+
+
+void padthv1_lv2::qapp_cleanup (void)
+{
+	if (g_qapp_instance && --g_qapp_refcount == 0) {
+		delete g_qapp_instance;
+		g_qapp_instance = nullptr;
+	}
+}
+
+
+QApplication *padthv1_lv2::qapp_instance (void)
+{
+	return g_qapp_instance;
+}
+
+
 //-------------------------------------------------------------------------
 // padthv1_lv2 - LV2 State interface.
 //
@@ -289,9 +409,11 @@ static LV2_State_Status padthv1_lv2_state_save ( LV2_Handle instance,
 	uint32_t flags, const LV2_Feature *const * /*features*/ )
 {
 	padthv1_lv2 *pPlugin = static_cast<padthv1_lv2 *> (instance);
-	if (pPlugin == NULL)
+	if (pPlugin == nullptr)
 		return LV2_STATE_ERR_UNKNOWN;
 
+	// Save state as XML chunk...
+	//
 	const uint32_t key = pPlugin->urid_map(PADTHV1_LV2_PREFIX "state");
 	if (key == 0)
 		return LV2_STATE_ERR_NO_PROPERTY;
@@ -307,9 +429,19 @@ static LV2_State_Status padthv1_lv2_state_save ( LV2_Handle instance,
 #endif
 
 	QDomDocument doc(PADTHV1_TITLE);
+	QDomElement eState = doc.createElement("state");
+
 	QDomElement eSamples = doc.createElement("samples");
 	padthv1_param::saveSamples(pPlugin, doc, eSamples);
-	doc.appendChild(eSamples);
+	eState.appendChild(eSamples);
+
+	if (pPlugin->isTuningEnabled()) {
+		QDomElement eTuning = doc.createElement("tuning");
+		padthv1_param::saveTuning(pPlugin, doc, eTuning);
+		eState.appendChild(eTuning);
+	}
+
+	doc.appendChild(eState);
 
 	const QByteArray data(doc.toByteArray());
 	const char *value = data.constData();
@@ -324,9 +456,11 @@ static LV2_State_Status padthv1_lv2_state_restore ( LV2_Handle instance,
 	uint32_t flags, const LV2_Feature *const * /*features*/ )
 {
 	padthv1_lv2 *pPlugin = static_cast<padthv1_lv2 *> (instance);
-	if (pPlugin == NULL)
+	if (pPlugin == nullptr)
 		return LV2_STATE_ERR_UNKNOWN;
 
+	// Retrieve state as XML chunk...
+	//
 	const uint32_t key = pPlugin->urid_map(PADTHV1_LV2_PREFIX "state");
 	if (key == 0)
 		return LV2_STATE_ERR_NO_PROPERTY;
@@ -351,14 +485,31 @@ static LV2_State_Status padthv1_lv2_state_restore ( LV2_Handle instance,
 	if ((flags & (LV2_STATE_IS_POD | LV2_STATE_IS_PORTABLE)) == 0)
 		return LV2_STATE_ERR_BAD_FLAGS;
 
-	if (value == NULL)
+	if (value == nullptr)
 		return LV2_STATE_ERR_UNKNOWN;
 
 	QDomDocument doc(PADTHV1_TITLE);
 	if (doc.setContent(QByteArray(value, size))) {
-		QDomElement eSamples = doc.documentElement();
-		if (eSamples.tagName() == "samples")
-			padthv1_param::loadSamples(pPlugin, eSamples);
+		QDomElement eState = doc.documentElement();
+	#if 1//PADTHV1_LV2_LEGACY
+		if (eState.tagName() == "samples")
+			padthv1_param::loadSamples(pPlugin, eState);
+		else
+	#endif
+		if (eState.tagName() == "state") {
+			for (QDomNode nChild = eState.firstChild();
+					!nChild.isNull();
+						nChild = nChild.nextSibling()) {
+				QDomElement eChild = nChild.toElement();
+				if (eChild.isNull())
+					continue;
+				if (eChild.tagName() == "samples")
+					padthv1_param::loadSamples(pPlugin, eChild);
+				else
+				if (eChild.tagName() == "tuning")
+					padthv1_param::loadTuning(pPlugin, eChild);
+			}
+		}
 	}
 
 	pPlugin->reset();
@@ -403,7 +554,7 @@ const LV2_Program_Descriptor *padthv1_lv2::get_program ( uint32_t index )
 		}
 	}
 
-	return NULL;
+	return nullptr;
 }
 
 void padthv1_lv2::select_program ( uint32_t bank, uint32_t program )
@@ -426,24 +577,54 @@ void padthv1_lv2::updatePreset ( bool /*bDirty*/ )
 }
 
 
-bool padthv1_lv2::worker_work ( const void *data, uint32_t /*size*/ )
+void padthv1_lv2::updateTuning (void)
 {
-	const padthv1_lv2_worker_message *mesg
-		= (const padthv1_lv2_worker_message *) data;
-
-	return (mesg->atom.type == m_urids.state_StateChanged);
+	if (m_schedule) {
+		padthv1_lv2_worker_message mesg;
+		mesg.atom.type = m_urids.tun1_update;
+		mesg.atom.size = 0; // nothing else matters.
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
+	}
 }
 
 
-bool padthv1_lv2::worker_response ( const void *data, uint32_t /*size*/ )
+bool padthv1_lv2::worker_work ( const void *data, uint32_t size )
 {
+	if (size != sizeof(padthv1_lv2_worker_message))
+		return false;
+
+	const padthv1_lv2_worker_message *mesg
+		= (const padthv1_lv2_worker_message *) data;
+
+	if (mesg->atom.type == m_urids.state_StateChanged)
+		return true;
+	else
+	if (mesg->atom.type == m_urids.tun1_update) {
+		padthv1::resetTuning();
+		return true;
+	}
+
+	return false;
+}
+
+
+bool padthv1_lv2::worker_response ( const void *data, uint32_t size )
+{
+	if (size != sizeof(padthv1_lv2_worker_message))
+		return false;
+
 	const padthv1_lv2_worker_message *mesg
 		= (const padthv1_lv2_worker_message *) data;
 
 	if (mesg->atom.type == m_urids.state_StateChanged)
 		return state_changed();
-	else
-		return false;
+
+#ifdef CONFIG_LV2_PATCH
+	return patch_put(m_ndelta, mesg->atom.type);
+#else
+	return true;
+#endif
 }
 
 
@@ -459,6 +640,60 @@ bool padthv1_lv2::state_changed (void)
 }
 
 
+#ifdef CONFIG_LV2_PATCH
+
+bool padthv1_lv2::patch_put ( uint32_t ndelta, uint32_t type )
+{
+	static char s_szNull[1] = {'\0'};
+
+	if (type == m_urids.patch_Put)
+		type = 0;
+
+	lv2_atom_forge_frame_time(&m_forge, ndelta);
+
+	LV2_Atom_Forge_Frame patch_frame;
+	lv2_atom_forge_object(&m_forge, &patch_frame, 0, m_urids.patch_Put);
+	lv2_atom_forge_key(&m_forge, m_urids.patch_body);
+
+	LV2_Atom_Forge_Frame body_frame;
+	lv2_atom_forge_object(&m_forge, &body_frame, 0, 0);
+
+	if (type == 0 || type == m_urids.p201_tuning_enabled) {
+		lv2_atom_forge_key(&m_forge, m_urids.p201_tuning_enabled);
+		lv2_atom_forge_bool(&m_forge, padthv1::isTuningEnabled());
+	}
+	if (type == 0 || type == m_urids.p202_tuning_refPitch) {
+		lv2_atom_forge_key(&m_forge, m_urids.p202_tuning_refPitch);
+		lv2_atom_forge_float(&m_forge, padthv1::tuningRefPitch());
+	}
+	if (type == 0 || type == m_urids.p203_tuning_refNote) {
+		lv2_atom_forge_key(&m_forge, m_urids.p203_tuning_refNote);
+		lv2_atom_forge_int(&m_forge, padthv1::tuningRefNote());
+	}
+	if (type == 0 || type == m_urids.p204_tuning_scaleFile) {
+		const char *pszScaleFile = padthv1::tuningScaleFile();
+		if (pszScaleFile == nullptr)
+			pszScaleFile = s_szNull;
+		lv2_atom_forge_key(&m_forge, m_urids.p204_tuning_scaleFile);
+		lv2_atom_forge_path(&m_forge, pszScaleFile, ::strlen(pszScaleFile) + 1);
+	}
+	if (type == 0 || type == m_urids.p205_tuning_keyMapFile) {
+		const char *pszKeyMapFile = padthv1::tuningKeyMapFile();
+		if (pszKeyMapFile == nullptr)
+			pszKeyMapFile = s_szNull;
+		lv2_atom_forge_key(&m_forge, m_urids.p205_tuning_keyMapFile);
+		lv2_atom_forge_path(&m_forge, pszKeyMapFile, ::strlen(pszKeyMapFile) + 1);
+	}
+
+	lv2_atom_forge_pop(&m_forge, &body_frame);
+	lv2_atom_forge_pop(&m_forge, &patch_frame);
+
+	return true;
+}
+
+#endif	// CONFIG_LV2_PATCH
+
+
 //-------------------------------------------------------------------------
 // padthv1_lv2 - LV2 desc.
 //
@@ -467,6 +702,8 @@ static LV2_Handle padthv1_lv2_instantiate (
 	const LV2_Descriptor *, double sample_rate, const char *,
 	const LV2_Feature *const *host_features )
 {
+	padthv1_lv2::qapp_instantiate();
+
 	return new padthv1_lv2(sample_rate, host_features);
 }
 
@@ -509,6 +746,8 @@ static void padthv1_lv2_cleanup ( LV2_Handle instance )
 	padthv1_lv2 *pPlugin = static_cast<padthv1_lv2 *> (instance);
 	if (pPlugin)
 		delete pPlugin;
+
+	padthv1_lv2::qapp_cleanup();
 }
 
 
@@ -521,7 +760,7 @@ static const LV2_Program_Descriptor *padthv1_lv2_programs_get_program (
 	if (pPlugin)
 		return pPlugin->get_program(index);
 	else
-		return NULL;
+		return nullptr;
 }
 
 static void padthv1_lv2_programs_select_program (
@@ -562,7 +801,7 @@ static LV2_Worker_Status padthv1_lv2_worker_response (
 	if (pSynth && pSynth->worker_response(data, size))
 		return LV2_WORKER_SUCCESS;
 	else
-		return LV2_WORKER_SUCCESS;
+		return LV2_WORKER_ERR_UNKNOWN;
 }
 
 
@@ -570,7 +809,7 @@ static const LV2_Worker_Interface padthv1_lv2_worker_interface =
 {
 	padthv1_lv2_worker_work,
 	padthv1_lv2_worker_response,
-	NULL
+	nullptr
 };
 
 
@@ -587,7 +826,7 @@ static const void *padthv1_lv2_extension_data ( const char *uri )
 	if (::strcmp(uri, LV2_STATE__interface) == 0)
 		return &padthv1_lv2_state_interface;
 
-	return NULL;
+	return nullptr;
 }
 
 
@@ -606,7 +845,7 @@ static const LV2_Descriptor padthv1_lv2_descriptor =
 
 LV2_SYMBOL_EXPORT const LV2_Descriptor *lv2_descriptor ( uint32_t index )
 {
-	return (index == 0 ? &padthv1_lv2_descriptor : NULL);
+	return (index == 0 ? &padthv1_lv2_descriptor : nullptr);
 }
 
 
