@@ -1,7 +1,7 @@
 // padthv1_lv2.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -24,7 +24,6 @@
 
 #include "padthv1.h"
 
-#include "lv2.h"
 #include "lv2/lv2plug.in/ns/ext/urid/urid.h"
 #include "lv2/lv2plug.in/ns/ext/atom/atom.h"
 #include "lv2/lv2plug.in/ns/ext/atom/forge.h"
@@ -40,6 +39,9 @@
 #include <QByteArray>
 #endif
 
+// Forward decls.
+class QApplication;
+
 
 //-------------------------------------------------------------------------
 // padthv1_lv2 - decl.
@@ -81,26 +83,39 @@ public:
 	bool worker_work(const void *data, uint32_t size);
 	bool worker_response(const void *data, uint32_t size);
 
+	static void qapp_instantiate();
+	static void qapp_cleanup();
+
+	static QApplication *qapp_instance();
+
 protected:
 
 	void updatePreset(bool bDirty);
+	void updateTuning();
 
 	bool state_changed();
 
+#ifdef CONFIG_LV2_PATCH
+	bool patch_put(uint32_t ndelta, uint32_t type = 0);
+#endif
+
 private:
 
 	LV2_URID_Map *m_urid_map;
 
 	struct lv2_urids
 	{
-		LV2_URID gen1_sample;
-		LV2_URID gen1_loop_start;
-		LV2_URID gen1_loop_end;
-		LV2_URID gen1_update;
+		LV2_URID p201_tuning_enabled;
+		LV2_URID p202_tuning_refPitch;
+		LV2_URID p203_tuning_refNote;
+		LV2_URID p204_tuning_scaleFile;
+		LV2_URID p205_tuning_keyMapFile;
+		LV2_URID tun1_update;
 		LV2_URID atom_Blank;
 		LV2_URID atom_Object;
 		LV2_URID atom_Float;
 		LV2_URID atom_Int;
+		LV2_URID atom_Bool;
 		LV2_URID atom_Path;
 		LV2_URID time_Position;
 		LV2_URID time_beatsPerMinute;
@@ -109,6 +124,14 @@ private:
 		LV2_URID bufsz_maxBlockLength;
 		LV2_URID bufsz_nominalBlockLength;
 		LV2_URID state_StateChanged;
+	#ifdef CONFIG_LV2_PATCH
+		LV2_URID patch_Get;
+		LV2_URID patch_Set;
+		LV2_URID patch_Put;
+		LV2_URID patch_body;
+		LV2_URID patch_property;
+		LV2_URID patch_value;
+	#endif
 	} m_urids;
 
 	LV2_Atom_Forge m_forge;
@@ -128,6 +151,9 @@ private:
 	LV2_Program_Descriptor m_program;
 	QByteArray m_aProgramName;
 #endif
+
+	static QApplication *g_qapp_instance;
+	static unsigned int  g_qapp_refcount;
 };
 
 
