@@ -1,7 +1,7 @@
 // padthv1_param.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -70,6 +70,7 @@ struct ParamInfo {
 	{ "GEN1_OCTAVE",   PARAM_FLOAT,   0.0f,  -4.0f,   4.0f }, // GEN1 Octave
 	{ "GEN1_TUNING",   PARAM_FLOAT,   0.0f,  -1.0f,   1.0f }, // GEN1 Tuning
 	{ "GEN1_ENVTIME",  PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // GEN1 Env.Time
+	{ "DCF1_ENABLED",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }, // DCF1 Enabled
 	{ "DCF1_CUTOFF",   PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCF1 Cutoff
 	{ "DCF1_RESO",     PARAM_FLOAT,   0.0f,   0.0f,   1.0f }, // DCF1 Resonance
 	{ "DCF1_TYPE",     PARAM_INT,     0.0f,   0.0f,   3.0f }, // DCF1 Type
@@ -79,6 +80,7 @@ struct ParamInfo {
 	{ "DCF1_DECAY",    PARAM_FLOAT,   0.2f,   0.0f,   1.0f }, // DCF1 Decay
 	{ "DCF1_SUSTAIN",  PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCF1 Sustain
 	{ "DCF1_RELEASE",  PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // DCF1 Release
+	{ "LFO1_ENABLED",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }, // LFO1 Enabled
 	{ "LFO1_SHAPE",    PARAM_INT,     1.0f,   0.0f,   4.0f }, // LFO1 Wave Shape
 	{ "LFO1_WIDTH",    PARAM_FLOAT,   1.0f,   0.0f,   1.0f }, // LFO1 Wave Width
 	{ "LFO1_BPM",      PARAM_FLOAT, 180.0f,   0.0f, 360.0f }, // LFO1 BPM
@@ -137,7 +139,10 @@ struct ParamInfo {
 	{ "REV1_FEEDB",    PARAM_FLOAT,   0.5f,   0.0f,   1.0f }, // Reverb Feedback
 	{ "REV1_WIDTH",    PARAM_FLOAT,   0.0f,  -1.0f,   1.0f }, // Reverb Width
 	{ "DYN1_COMPRESS", PARAM_BOOL,    0.0f,   0.0f,   1.0f }, // Dynamic Compressor
-	{ "DYN1_LIMITER",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }  // Dynamic Limiter
+	{ "DYN1_LIMITER",  PARAM_BOOL,    1.0f,   0.0f,   1.0f }, // Dynamic Limiter
+
+	{ "KEY1_LOW",      PARAM_INT,     0.0f,   0.0f, 127.0f }, // Keyboard Low
+	{ "KEY1_HIGH",     PARAM_INT,   127.0f,   0.0f, 127.0f }  // Keyboard High
 };
 
 
@@ -153,6 +158,25 @@ float padthv1_param::paramDefaultValue ( padthv1::ParamIndex index )
 }
 
 
+float padthv1_param::paramSafeValue ( padthv1::ParamIndex index, float fValue )
+{
+	const ParamInfo& param = padthv1_params[index];
+
+	if (param.type == PARAM_BOOL)
+		return (fValue > 0.5f ? 1.0f : 0.0f);
+
+	if (fValue < param.min)
+		return param.min;
+	if (fValue > param.max)
+		return param.max;
+
+	if (param.type == PARAM_INT)
+		return ::rintf(fValue);
+	else
+		return fValue;
+}
+
+
 float padthv1_param::paramValue ( padthv1::ParamIndex index, float fScale )
 {
 	const ParamInfo& param = padthv1_params[index];
@@ -191,12 +215,11 @@ bool padthv1_param::paramFloat ( padthv1::ParamIndex index )
 }
 
 
-
 // Preset serialization methods.
 bool padthv1_param::loadPreset (
 	padthv1 *pSynth, const QString& sFilename )
 {
-	if (pSynth == NULL)
+	if (pSynth == nullptr)
 		return false;
 
 	QFileInfo fi(sFilename);
@@ -217,6 +240,11 @@ bool padthv1_param::loadPreset (
 	if (!file.open(QIODevice::ReadOnly))
 		return false;
 
+	const bool running = pSynth->running(false);
+
+	pSynth->setTuningEnabled(false);
+	pSynth->reset();
+
 	static QHash<QString, padthv1::ParamIndex> s_hash;
 	if (s_hash.isEmpty()) {
 		for (uint32_t i = 0; i < padthv1::NUM_PARAMS; ++i) {
@@ -256,7 +284,8 @@ bool padthv1_param::loadPreset (
 								index = s_hash.value(sName);
 							}
 							const float fValue = eParam.text().toFloat();
-							pSynth->setParamValue(index, fValue);
+							pSynth->setParamValue(index,
+								padthv1_param::paramSafeValue(index, fValue));
 						}
 					}
 				}
@@ -264,13 +293,19 @@ bool padthv1_param::loadPreset (
 				if (eChild.tagName() == "samples") {
 					padthv1_param::loadSamples(pSynth, eChild);
 				}
+				else
+				if (eChild.tagName() == "tuning") {
+					padthv1_param::loadTuning(pSynth, eChild);
+				}
 			}
 		}
 	}
 
 	file.close();
 
+	pSynth->stabilize();
 	pSynth->reset();
+	pSynth->running(running);
 
 	QDir::setCurrent(currentDir.absolutePath());
 
@@ -281,9 +316,11 @@ bool padthv1_param::loadPreset (
 bool padthv1_param::savePreset (
 	padthv1 *pSynth, const QString& sFilename, bool bSymLink )
 {
-	if (pSynth == NULL)
+	if (pSynth == nullptr)
 		return false;
 
+	pSynth->stabilize();
+
 	const QFileInfo fi(sFilename);
 	const QDir currentDir(QDir::current());
 	QDir::setCurrent(fi.absolutePath());
@@ -309,6 +346,12 @@ bool padthv1_param::savePreset (
 	}
 	ePreset.appendChild(eParams);
 
+	if (pSynth->isTuningEnabled()) {
+		QDomElement eTuning = doc.createElement("tuning");
+		padthv1_param::saveTuning(pSynth, doc, eTuning, bSymLink);
+		ePreset.appendChild(eTuning);
+	}
+
 	doc.appendChild(ePreset);
 
 	QFile file(fi.filePath());
@@ -328,7 +371,7 @@ bool padthv1_param::savePreset (
 void padthv1_param::loadSamples (
 	padthv1 *pSynth, const QDomElement& eSamples )
 {
-	if (pSynth == NULL)
+	if (pSynth == nullptr)
 		return;
 
 	QHash<int, padthv1_sample *> list;
@@ -343,8 +386,8 @@ void padthv1_param::loadSamples (
 			continue;
 		if (eSample.tagName() == "sample") {
 			const int index = eSample.attribute("index").toInt();
-			padthv1_sample *sample = list.value(index, NULL);
-			if (sample == NULL)
+			padthv1_sample *sample = list.value(index, nullptr);
+			if (sample == nullptr)
 				continue;
 			sample->reset_nh(eSample.attribute("nh").toUInt());
 			for (QDomNode nChild = eSample.firstChild();
@@ -376,7 +419,7 @@ void padthv1_param::loadSamples (
 void padthv1_param::saveSamples (
 	padthv1 *pSynth, QDomDocument& doc, QDomElement& eSamples, bool /*bSymLink*/ )
 {
-	if (pSynth == NULL)
+	if (pSynth == nullptr)
 		return;
 
 	QHash<int, padthv1_sample *> list;
@@ -387,7 +430,7 @@ void padthv1_param::saveSamples (
 	const QHash<int, padthv1_sample *>::ConstIterator& iter_end = list.constEnd();
 	for ( ; iter != iter_end; ++iter) {
 		padthv1_sample *sample = iter.value();
-		if (sample == NULL)
+		if (sample == nullptr)
 			continue;
 		const int index = iter.key();
 		QDomElement eSample = doc.createElement("sample");
@@ -407,4 +450,126 @@ void padthv1_param::saveSamples (
 }
 
 
+// Tuning serialization methods.
+void padthv1_param::loadTuning (
+	padthv1 *pSynth, const QDomElement& eTuning )
+{
+	if (pSynth == nullptr)
+		return;
+
+	pSynth->setTuningEnabled(eTuning.attribute("enabled").toInt() > 0);
+
+	for (QDomNode nChild = eTuning.firstChild();
+			!nChild.isNull();
+				nChild = nChild.nextSibling()) {
+		QDomElement eChild = nChild.toElement();
+		if (eChild.isNull())
+			continue;
+		if (eChild.tagName() == "enabled") {
+			pSynth->setTuningEnabled(eChild.text().toInt() > 0);
+		}
+		if (eChild.tagName() == "ref-pitch") {
+			pSynth->setTuningRefPitch(eChild.text().toFloat());
+		}
+		else
+		if (eChild.tagName() == "ref-note") {
+			pSynth->setTuningRefNote(eChild.text().toInt());
+		}
+		else
+		if (eChild.tagName() == "scale-file") {
+			const QString& sScaleFile
+				= eChild.text();
+			const QByteArray aScaleFile
+				= padthv1_param::loadFilename(sScaleFile).toUtf8();
+			pSynth->setTuningScaleFile(aScaleFile.constData());
+		}
+		else
+		if (eChild.tagName() == "keymap-file") {
+			const QString& sKeyMapFile
+				= eChild.text();
+			const QByteArray aKeyMapFile
+				= padthv1_param::loadFilename(sKeyMapFile).toUtf8();
+			pSynth->setTuningScaleFile(aKeyMapFile.constData());
+		}
+	}
+
+	// Consolidate tuning state...
+	pSynth->resetTuning();
+}
+
+
+void padthv1_param::saveTuning (
+	padthv1 *pSynth, QDomDocument& doc, QDomElement& eTuning, bool bSymLink )
+{
+	if (pSynth == nullptr)
+		return;
+
+	eTuning.setAttribute("enabled", int(pSynth->isTuningEnabled()));
+
+	QDomElement eRefPitch = doc.createElement("ref-pitch");
+	eRefPitch.appendChild(doc.createTextNode(
+		QString::number(pSynth->tuningRefPitch())));
+	eTuning.appendChild(eRefPitch);
+
+	QDomElement eRefNote = doc.createElement("ref-note");
+	eRefNote.appendChild(doc.createTextNode(
+		QString::number(pSynth->tuningRefNote())));
+	eTuning.appendChild(eRefNote);
+
+	const char *pszScaleFile = pSynth->tuningScaleFile();
+	if (pszScaleFile) {
+		const QString& sScaleFile
+			= QString::fromUtf8(pszScaleFile);
+		if (!sScaleFile.isEmpty()) {
+			QDomElement eScaleFile = doc.createElement("scale-file");
+			eScaleFile.appendChild(doc.createTextNode(
+				QDir::current().relativeFilePath(
+					padthv1_param::saveFilename(sScaleFile, bSymLink))));
+			eTuning.appendChild(eScaleFile);
+		}
+	}
+
+	const char *pszKeyMapFile = pSynth->tuningKeyMapFile();
+	if (pszKeyMapFile) {
+		const QString& sKeyMapFile
+			= QString::fromUtf8(pszKeyMapFile);
+		if (!sKeyMapFile.isEmpty()) {
+			QDomElement eKeyMapFile = doc.createElement("keymap-file");
+			eKeyMapFile.appendChild(doc.createTextNode(
+				QDir::current().relativeFilePath(
+					padthv1_param::saveFilename(sKeyMapFile, bSymLink))));
+			eTuning.appendChild(eKeyMapFile);
+		}
+	}
+}
+
+
+// Load/save and convert canonical/absolute filename helpers.
+QString padthv1_param::loadFilename ( const QString& sFilename )
+{
+	QFileInfo fi(sFilename);
+	if (fi.isSymLink())
+		fi.setFile(fi.symLinkTarget());
+	return fi.canonicalFilePath();
+}
+
+
+QString padthv1_param::saveFilename ( const QString& sFilename, bool bSymLink )
+{
+	QFileInfo fi(sFilename);
+	if (bSymLink && fi.absolutePath() != QDir::current().absolutePath()) {
+		const QString& sPath = fi.absoluteFilePath();
+		const QString& sName = fi.baseName();
+		const QString& sExt  = fi.completeSuffix();
+		const QString& sLink = sName
+			+ '-' + QString::number(qHash(sPath), 16)
+			+ '.' + sExt;
+		QFile(sPath).link(sLink);
+		fi.setFile(QDir::current(), sLink);
+	}
+	else if (fi.isSymLink()) fi.setFile(fi.symLinkTarget());
+	return fi.absoluteFilePath();
+}
+
+
 // end of padthv1_param.cpp
