@@ -1,7 +1,7 @@
 // padthv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,6 +28,8 @@
 #include "padthv1widget_config.h"
 #include "padthv1widget_control.h"
 
+#include "padthv1widget_keybd.h"
+
 #include "padthv1_controls.h"
 #include "padthv1_programs.h"
 
@@ -40,6 +42,10 @@
 #include <QShowEvent>
 #include <QHideEvent>
 
+#include <math.h>
+
+#include <random>
+
 
 //-------------------------------------------------------------------------
 // padthv1widget - impl.
@@ -51,7 +57,7 @@ padthv1widget::padthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 {
 	Q_INIT_RESOURCE(padthv1);
 
-#if QT_VERSION >= 0x050000
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 	// HACK: Dark themes grayed/disabled color group fix...
 	QPalette pal;
 	if (pal.base().color().value() < 0x7f) {
@@ -73,7 +79,7 @@ padthv1widget::padthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.setupUi(this);
 
 	// Init sched notifier.
-	m_sched_notifier = NULL;
+	m_sched_notifier = nullptr;
 
 	// Init swapable params A/B to default.
 	for (uint32_t i = 0; i < padthv1::NUM_PARAMS; ++i)
@@ -89,7 +95,7 @@ padthv1widget::padthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	// Note names.
 	QStringList notes;
 	for (int note = 0; note < 128; ++note)
-		notes << noteName(note);
+		notes << padthv1_ui::noteName(note).remove(QRegExp("/\\S+"));
 
 	m_ui.Gen1Sample1Knob->setScale(1000.0f);
 	m_ui.Gen1Sample1Knob->insertItems(0, notes);
@@ -296,6 +302,7 @@ padthv1widget::padthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	setParamKnob(padthv1::GEN1_ENVTIME, m_ui.Gen1EnvTimeKnob);
 
 	// DCF1
+	setParamKnob(padthv1::DCF1_ENABLED, m_ui.Dcf1GroupBox->param());
 	setParamKnob(padthv1::DCF1_CUTOFF,   m_ui.Dcf1CutoffKnob);
 	setParamKnob(padthv1::DCF1_RESO,     m_ui.Dcf1ResoKnob);
 	setParamKnob(padthv1::DCF1_TYPE,     m_ui.Dcf1TypeKnob);
@@ -356,6 +363,7 @@ padthv1widget::padthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		m_ui.Dcf1Env, SLOT(setRelease(float)));
 
 	// LFO1
+	setParamKnob(padthv1::LFO1_ENABLED, m_ui.Lfo1GroupBox->param());
 	setParamKnob(padthv1::LFO1_SHAPE,   m_ui.Lfo1ShapeKnob);
 	setParamKnob(padthv1::LFO1_WIDTH,   m_ui.Lfo1WidthKnob);
 	setParamKnob(padthv1::LFO1_BPM,     m_ui.Lfo1BpmKnob);
@@ -464,6 +472,8 @@ padthv1widget::padthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	setParamKnob(padthv1::OUT1_FXSEND,  m_ui.Out1FxSendKnob);
 	setParamKnob(padthv1::OUT1_VOLUME,  m_ui.Out1VolumeKnob);
 
+	// Make status-bar keyboard range active.
+	m_ui.StatusBar->keybd()->setNoteRange(true);
 
 	// Effects
 	setParamKnob(padthv1::CHO1_WET,   m_ui.Cho1WetKnob);
@@ -514,6 +524,11 @@ padthv1widget::padthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(resetPresetFile()),
 		SLOT(resetParams()));
 
+	// Randomize params...
+	QObject::connect(m_ui.RandomParamsButton,
+		SIGNAL(clicked()),
+		SLOT(randomParams()));
+
 	// Swap params A/B
 	QObject::connect(m_ui.SwapParamsAButton,
 		SIGNAL(toggled(bool)),
@@ -533,6 +548,14 @@ padthv1widget::padthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	QObject::connect(m_ui.TabBar, SIGNAL(currentChanged(int)),
 		m_ui.StackedWidget, SLOT(setCurrentIndex(int)));
 
+	// Direct status-bar keyboard input
+	QObject::connect(m_ui.StatusBar->keybd(),
+		SIGNAL(noteOnClicked(int, int)),
+		SLOT(directNoteOn(int, int)));
+	QObject::connect(m_ui.StatusBar->keybd(),
+		SIGNAL(noteRangeChanged()),
+		SLOT(noteRangeChanged()));
+
 	// Menu actions
 	QObject::connect(m_ui.helpConfigureAction,
 		SIGNAL(triggered(bool)),
@@ -579,7 +602,7 @@ void padthv1widget::openSchedNotifier (void)
 		return;
 
 	padthv1_ui *pSynthUi = ui_instance();
-	if (pSynthUi == NULL)
+	if (pSynthUi == nullptr)
 		return;
 
 	m_sched_notifier = new padthv1widget_sched(pSynthUi->instance(), this);
@@ -596,7 +619,7 @@ void padthv1widget::closeSchedNotifier (void)
 {
 	if (m_sched_notifier) {
 		delete m_sched_notifier;
-		m_sched_notifier = NULL;
+		m_sched_notifier = nullptr;
 	}
 
 	padthv1_ui *pSynthUi = ui_instance();
@@ -643,19 +666,18 @@ void padthv1widget::setParamKnob ( padthv1::ParamIndex index, padthv1widget_para
 
 padthv1widget_param *padthv1widget::paramKnob ( padthv1::ParamIndex index ) const
 {
-	return m_paramKnobs.value(index, NULL);
+	return m_paramKnobs.value(index, nullptr);
 }
 
 
 // Param port accessors.
-void padthv1widget::setParamValue (
-	padthv1::ParamIndex index, float fValue, bool bDefault )
+void padthv1widget::setParamValue ( padthv1::ParamIndex index, float fValue )
 {
 	++m_iUpdate;
 
 	padthv1widget_param *pParam = paramKnob(index);
 	if (pParam)
-		pParam->setValue(fValue, bDefault);
+		pParam->setValue(fValue);
 
 	updateParamEx(index, fValue);
 
@@ -702,15 +724,44 @@ void padthv1widget::paramChanged ( float fValue )
 void padthv1widget::updateParamEx ( padthv1::ParamIndex index, float fValue )
 {
 	padthv1_ui *pSynthUi = ui_instance();
-	if (pSynthUi == NULL)
+	if (pSynthUi == nullptr)
 		return;
 
 	++m_iUpdate;
 
 	switch (index) {
-	case padthv1::DCF1_SLOPE:
-		m_ui.Dcf1TypeKnob->setEnabled(int(fValue) != 3); // !Formant
+	case padthv1::DCF1_ENABLED:
+		if (m_ui.Lfo1GroupBox->isChecked()) {
+			const bool bDcf1Enabled = (fValue > 0.5f);
+			m_ui.Lfo1CutoffKnob->setEnabled(bDcf1Enabled);
+			m_ui.Lfo1ResoKnob->setEnabled(bDcf1Enabled);
+		}
 		break;
+	case padthv1::LFO1_ENABLED:
+		if (fValue > 0.5f) {
+			const bool bDcf1Enabled = m_ui.Dcf1GroupBox->isChecked();
+			m_ui.Lfo1CutoffKnob->setEnabled(bDcf1Enabled);
+			m_ui.Lfo1ResoKnob->setEnabled(bDcf1Enabled);
+		}
+		break;
+	case padthv1::DCF1_SLOPE:
+		if (m_ui.Dcf1GroupBox->isChecked())
+			m_ui.Dcf1TypeKnob->setEnabled(int(fValue) != 3); // !Formant
+		break;
+	case padthv1::LFO1_SHAPE:
+		m_ui.Lfo1Wave->setWaveShape(fValue);
+		break;
+	case padthv1::KEY1_LOW:
+		m_ui.StatusBar->keybd()->setNoteLow(int(fValue));
+		break;
+	case padthv1::KEY1_HIGH:
+		m_ui.StatusBar->keybd()->setNoteHigh(int(fValue));
+		break;
+	case padthv1::DEF1_VELOCITY: {
+		const int vel = int(79.375f * fValue + 47.625f) & 0x7f;
+		m_ui.StatusBar->keybd()->setVelocity(vel);
+		break;
+	}
 	default:
 		break;
 	}
@@ -726,7 +777,7 @@ void padthv1widget::updateSchedParam ( padthv1::ParamIndex index, float fValue )
 
 	padthv1widget_param *pParam = paramKnob(index);
 	if (pParam) {
-		pParam->setValue(fValue, false);
+		pParam->setValue(fValue);
 		updateParam(index, fValue);
 		updateParamEx(index, fValue);
 		m_ui.StatusBar->showMessage(QString("%1: %2")
@@ -743,7 +794,7 @@ void padthv1widget::updateSchedParam ( padthv1::ParamIndex index, float fValue )
 void padthv1widget::resetParams (void)
 {
 	padthv1_ui *pSynthUi = ui_instance();
-	if (pSynthUi == NULL)
+	if (pSynthUi == nullptr)
 		return;
 
 	pSynthUi->reset();
@@ -767,6 +818,64 @@ void padthv1widget::resetParams (void)
 }
 
 
+// Randomize params (partial).
+void padthv1widget::randomParams (void)
+{
+	padthv1_ui *pSynthUi = ui_instance();
+	if (pSynthUi == nullptr)
+		return;
+
+	float p = 1.0f;
+
+	padthv1_config *pConfig = padthv1_config::getInstance();
+	if (pConfig)
+		p = 0.01f * pConfig->fRandomizePercent;
+
+	if (QMessageBox::warning(this,
+		tr("Warning"),
+		tr("About to randomize current parameter values:\n\n"
+		"-/+ %1%.\n\n"
+		"Are you sure?").arg(100.0f * p),
+		QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Cancel)
+		return;
+
+	std::default_random_engine re(::time(nullptr));
+
+	for (uint32_t i = 0; i < padthv1::NUM_PARAMS; ++i) {
+		const padthv1::ParamIndex index = padthv1::ParamIndex(i);
+		// Filter out some non-randomizable parameters!...
+		if (index == padthv1::GEN1_OCTAVE  ||
+		    index == padthv1::GEN1_TUNING  ||
+		    index == padthv1::GEN1_ENVTIME ||
+			index == padthv1::DCF1_ENABLED ||
+			index == padthv1::LFO1_ENABLED) 
+			continue;
+		if (index >= padthv1::OUT1_WIDTH)
+			break;
+		padthv1widget_param *pParam = paramKnob(index);
+		if (pParam) {
+			std::normal_distribution<float> nd;
+			const float q = p * (pParam->maximum() - pParam->minimum());
+			float fValue = pParam->value();
+			if (padthv1_param::paramFloat(index))
+				fValue += 0.5f * q * nd(re);
+			else
+				fValue = std::round(fValue + q * nd(re));
+			if (fValue < pParam->minimum())
+				fValue = pParam->minimum();
+			else
+			if (fValue > pParam->maximum())
+				fValue = pParam->maximum();
+			setParamValue(index, fValue);
+			updateParam(index, fValue);
+		}
+	}
+
+	m_ui.StatusBar->showMessage(tr("Randomize"), 5000);
+	updateDirtyPreset(true);
+}
+
+
 // Swap params A/B.
 void padthv1widget::swapParams ( bool bOn )
 {
@@ -776,7 +885,6 @@ void padthv1widget::swapParams ( bool bOn )
 #ifdef CONFIG_DEBUG
 	qDebug("padthv1widget::swapParams(%d)", int(bOn));
 #endif
-//	resetParamKnobs();
 
 	for (uint32_t i = 0; i < padthv1::NUM_PARAMS; ++i) {
 		const padthv1::ParamIndex index = padthv1::ParamIndex(i);
@@ -817,7 +925,7 @@ void padthv1widget::updateParamValues (void)
 		const float fValue = (pSynthUi
 			? pSynthUi->paramValue(index)
 			: padthv1_param::paramDefaultValue(index));
-		setParamValue(index, fValue, true);
+		setParamValue(index, fValue);
 		updateParam(index, fValue);
 	//	updateParamEx(index, fValue);
 		m_params_ab[i] = fValue;
@@ -833,7 +941,7 @@ void padthv1widget::resetParamValues (void)
 	for (uint32_t i = 0; i < padthv1::NUM_PARAMS; ++i) {
 		const padthv1::ParamIndex index = padthv1::ParamIndex(i);
 		const float fValue = padthv1_param::paramDefaultValue(index);
-		setParamValue(index, fValue, true);
+		setParamValue(index, fValue);
 		updateParam(index, fValue);
 	//	updateParamEx(index, fValue);
 		m_params_ab[i] = fValue;
@@ -925,7 +1033,7 @@ void padthv1widget::savePreset ( const QString& sFilename )
 void padthv1widget::clearSample ( int sid )
 {
 	padthv1_ui *pSynthUi = ui_instance();
-	if (pSynthUi == NULL)
+	if (pSynthUi == nullptr)
 		return;
 
 	if (sid & 1)
@@ -939,7 +1047,7 @@ void padthv1widget::clearSample ( int sid )
 void padthv1widget::updateSample ( int sid )
 {
 	padthv1_ui *pSynthUi = ui_instance();
-	if (pSynthUi == NULL)
+	if (pSynthUi == nullptr)
 		return;
 
 	if (sid & 1)
@@ -949,15 +1057,6 @@ void padthv1widget::updateSample ( int sid )
 }
 
 
-// MIDI note/octave name helper (static).
-QString padthv1widget::noteName ( int note )
-{
-	static const char *notes[] =
-		{ "C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B" };
-	return QString("%1 %2").arg(notes[note % 12]).arg((note / 12) - 1);
-}
-
-
 // Dirty close prompt,
 bool padthv1widget::queryClose (void)
 {
@@ -968,6 +1067,7 @@ bool padthv1widget::queryClose (void)
 // Preset status updater.
 void padthv1widget::updateLoadPreset ( const QString& sPreset )
 {
+	resetParamKnobs();
 	updateParamValues();
 
 	m_ui.Preset->setPreset(sPreset);
@@ -980,15 +1080,21 @@ void padthv1widget::updateLoadPreset ( const QString& sPreset )
 void padthv1widget::updateSchedNotify ( int stype, int sid )
 {
 	padthv1_ui *pSynthUi = ui_instance();
-	if (pSynthUi == NULL)
+	if (pSynthUi == nullptr)
 		return;
 
-#ifdef CONFIG_DEBUG
+#ifdef CONFIG_DEBUG_0
 	qDebug("padthv1widget::updateSchedNotify(%d, 0x%04x)", stype, sid);
 #endif
 
 	switch (padthv1_sched::Type(stype)) {
 	case padthv1_sched::MidiIn:
+		if (sid >= 0) {
+			const int key = (sid & 0x7f);
+			const int vel = (sid >> 7) & 0x7f;
+			m_ui.StatusBar->midiInNote(key, vel);
+		}
+		else
 		if (pSynthUi->midiInCount() > 0) {
 			m_ui.StatusBar->midiInLed(true);
 			QTimer::singleShot(200, this, SLOT(midiInLedTimeout()));
@@ -1016,13 +1122,52 @@ void padthv1widget::updateSchedNotify ( int stype, int sid )
 	}
 	case padthv1_sched::Sample:
 		updateSample(sid);
-		// fall thru...
+		if (sid > 2) {
+			updateParamValues();
+			resetParamKnobs();
+			updateDirtyPreset(false);
+		}
+		// Fall thru...
 	default:
 		break;
 	}
 }
 
 
+// Direct note-on/off slot.
+void padthv1widget::directNoteOn ( int iNote, int iVelocity )
+{
+#ifdef CONFIG_DEBUG
+	qDebug("padthv1widget::directNoteOn(%d, %d)", iNote, iVelocity);
+#endif
+
+	padthv1_ui *pSynthUi = ui_instance();
+	if (pSynthUi)
+		pSynthUi->directNoteOn(iNote, iVelocity); // note-on!
+}
+
+
+// Keyboard note range change.
+void padthv1widget::noteRangeChanged (void)
+{
+	const int iNoteLow  = m_ui.StatusBar->keybd()->noteLow();
+	const int iNoteHigh = m_ui.StatusBar->keybd()->noteHigh();
+
+#ifdef CONFIG_DEBUG
+	qDebug("padthv1widget::noteRangeChanged(%d, %d)", iNoteLow, iNoteHigh);
+#endif
+
+	updateParam(padthv1::KEY1_LOW,  float(iNoteLow));
+	updateParam(padthv1::KEY1_HIGH, float(iNoteHigh));
+
+	m_ui.StatusBar->showMessage(QString("KEY Low: %1 (%2) High: %3 (%4)")
+		.arg(padthv1_ui::noteName(iNoteLow)).arg(iNoteLow)
+		.arg(padthv1_ui::noteName(iNoteHigh)).arg(iNoteHigh), 5000);
+
+	updateDirtyPreset(true);
+}
+
+
 // MIDI In LED timeout.
 void padthv1widget::midiInLedTimeout (void)
 {
@@ -1034,7 +1179,7 @@ void padthv1widget::midiInLedTimeout (void)
 void padthv1widget::helpConfigure (void)
 {
 	padthv1_ui *pSynthUi = ui_instance();
-	if (pSynthUi == NULL)
+	if (pSynthUi == nullptr)
 		return;
 
 	padthv1widget_config(pSynthUi, this).exec();
@@ -1085,7 +1230,7 @@ void padthv1widget::helpAbout (void)
 	sText += "</small>";
 	sText += "</p>\n";
 
-	QMessageBox::about(this, tr("About") + " " PADTHV1_TITLE, sText);
+	QMessageBox::about(this, tr("About"), sText);
 }
 
 
@@ -1113,15 +1258,15 @@ void padthv1widget::paramContextMenu ( const QPoint& pos )
 {
 	padthv1widget_param *pParam
 		= qobject_cast<padthv1widget_param *> (sender());
-	if (pParam == NULL)
+	if (pParam == nullptr)
 		return;
 
 	padthv1_ui *pSynthUi = ui_instance();
-	if (pSynthUi == NULL)
+	if (pSynthUi == nullptr)
 		return;
 
 	padthv1_controls *pControls = pSynthUi->controls();
-	if (pControls == NULL)
+	if (pControls == nullptr)
 		return;
 
 	if (!pControls->enabled())
