@@ -1,7 +1,7 @@
 // padthv1widget_jack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -21,6 +21,8 @@
 
 #include "padthv1widget_jack.h"
 
+#include "padthv1widget_palette.h"
+
 #include "padthv1_jack.h"
 
 #ifdef CONFIG_NSM
@@ -43,7 +45,7 @@
 #endif
 #endif
 
-#if QT_VERSION < 0x050000
+#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
 #define CONFIG_PLUGINSDIR CONFIG_LIBDIR "/qt4/plugins"
 #else
 #define CONFIG_PLUGINSDIR CONFIG_LIBDIR "/qt5/plugins"
@@ -58,29 +60,42 @@
 padthv1widget_jack::padthv1widget_jack ( padthv1_jack *pSynth )
 	: padthv1widget(), m_pSynth(pSynth)
 	#ifdef CONFIG_NSM
-		, m_pNsmClient(NULL)
+		, m_pNsmClient(nullptr)
 	#endif
 {
 	// Special style paths...
 	if (QDir(CONFIG_PLUGINSDIR).exists())
 		QApplication::addLibraryPath(CONFIG_PLUGINSDIR);
 
-	// Custom style theme...
+	// Custom color/style themes...
 	padthv1_config *pConfig = padthv1_config::getInstance();
-	if (pConfig && !pConfig->sCustomStyleTheme.isEmpty())
-		QApplication::setStyle(QStyleFactory::create(pConfig->sCustomStyleTheme));
+	if (pConfig) {
+		if (!pConfig->sCustomColorTheme.isEmpty()) {
+			QPalette pal;
+			if (padthv1widget_palette::namedPalette(
+					pConfig, pConfig->sCustomColorTheme, pal))
+				QApplication::setPalette(pal);
+		}
+		if (!pConfig->sCustomStyleTheme.isEmpty()) {
+			QApplication::setStyle(
+				QStyleFactory::create(pConfig->sCustomStyleTheme));
+		}
+	}
 
 	// Initialize (user) interface stuff...
 	m_pSynthUi = new padthv1_ui(m_pSynth, false);
 
-	// May initialize the scheduler/work notifier.
-	openSchedNotifier();
+	// Initialise preset stuff...
+	clearPreset();
 
-	// Initialize preset stuff...
-	// initPreset();
+	// Initial update, always...
 	updateSample();
 
-	updateParamValues();
+	resetParamValues();
+	resetParamKnobs();
+
+	// May initialize the scheduler/work notifier.
+	openSchedNotifier();
 }
 
 
@@ -103,10 +118,6 @@ padthv1_ui *padthv1widget_jack::ui_instance (void) const
 void padthv1widget_jack::setNsmClient ( padthv1_nsm *pNsmClient )
 {
 	m_pNsmClient = pNsmClient;
-
-	padthv1_config *pConfig = padthv1_config::getInstance();
-	if (pConfig)
-		pConfig->bDontUseNativeDialogs = true;
 }
 
 padthv1_nsm *padthv1widget_jack::nsmClient (void) const
@@ -132,7 +143,7 @@ void padthv1widget_jack::updateDirtyPreset ( bool bDirtyPreset )
 
 #ifdef CONFIG_NSM
 	if (m_pNsmClient && m_pNsmClient->is_active())
-		m_pNsmClient->dirty(bDirtyPreset);
+		m_pNsmClient->dirty(true); // as far as NSM goes, we're always filthy!
 #endif
 }
 
@@ -160,7 +171,7 @@ void padthv1widget_jack::closeEvent ( QCloseEvent *pCloseEvent )
 // Optional GUI handlers.
 void padthv1widget_jack::showEvent ( QShowEvent *pShowEvent )
 {
-	QWidget::showEvent(pShowEvent);
+	padthv1widget::showEvent(pShowEvent);
 
 	if (m_pNsmClient)
 		m_pNsmClient->visible(true);
@@ -171,7 +182,7 @@ void padthv1widget_jack::hideEvent ( QHideEvent *pHideEvent )
 	if (m_pNsmClient)
 		m_pNsmClient->visible(false);
 
-	QWidget::hideEvent(pHideEvent);
+	padthv1widget::hideEvent(pHideEvent);
 }
 
 #endif	// CONFIG_NSM
